@echo off
REM
REM BEGIN COPYRIGHT
REM *************************************************************************
REM Licensed Materials - Property of IBM 
REM 5724-L01, 5655-N53, 5724-I82, 5655-R15
REM (C) Copyright IBM Corporation 2006. All rights reserved. 
REM US Government Users Restricted Rights - Use, duplication, or disclosure
REM restricted by GSA ADP Schedule Contract with IBM Corp.
REM *************************************************************************
REM END COPYRIGHT
REM 
REM -- Description:   Configure Oracle Database environment for CommonDB during Migrtaion based 
REM --			      Migrating WBI version
REM --                This file for use on Windows systems
REM --                This batch file updates CommonDB database 
REM --
REM -- Usage: 
REM --                upgradeSchema.bat  <product_version>  <dbName> <dbuser>
REM -- Example:
REM --                upgradeSchema.bat
REM --                upgradeSchema.bat 602 orcl dbCommonUserId  
REM --                password will be prompted
REM -------------------------------------------------------------------------------------------
setlocal

set PRODUCT_VERSION=%1
set DB_NAME=%2
set DB_USER=%3
		
if  not "%1" == ""  (
  if not "%2"==""  (
  	if not "%3"==""  (
		goto INPUT_PRODUCT_VERSION
  	) else (
  	  echo "One or more parameters are missing..."
	  goto PRINT_USAGE
	)
  ) else (
  	echo "One or more parameters are missing..."
	goto PRINT_USAGE
  )
) 

:INPUT_PRODUCT_VERSION
if "%PRODUCT_VERSION%"==""  (
  set /p PRODUCT_VERSION="Enter WBI version that you migrtated [602 | 610 |612]:": 
  goto INPUT_PRODUCT_VERSION
)

:INPUT_DBNAME
if "%DB_NAME%"==""  (
  set /p DB_NAME="Enter Oracle SID[orcl]:":  
  goto INPUT_DBNAME
)

:INPUT_USERNAME
if "%DB_USER%"==""  (
  set /p DB_USER="Enter Oracle user[dbCommonUserId]:":
  goto INPUT_USERNAME
)

 
set ATSTRING=@

dir /B | findstr "upgradeSchema%PRODUCT_VERSION%_" > %PRODUCT_VERSION%.log


for /F "tokens=1* delims=" %%v in ( %PRODUCT_VERSION%.log) do (
echo %ATSTRING%%%v >> %PRODUCT_VERSION%.sql
)

echo quit >> %PRODUCT_VERSION%.sql

del %PRODUCT_VERSION%.log

call sqlplus %DB_USER%@%DB_NAME% @%PRODUCT_VERSION%.sql

del  %PRODUCT_VERSION%.sql

if not %ERRORLEVEL%==0 (
    goto PRINT_USAGE
) else (
    goto END 
)

:PRINT_USAGE

REM Print usage messages
echo.
echo ======================================================================
echo Usage: upgradeSchema.bat ^<product_version^>  ^<dbName^> ^<dbuser^>
echo.
echo Parameters:
echo product_version   -  Specify the migration version ie., 602 or 610 or 612 
echo dbname            -  Oracle SID^(dbName^) ie., orcl or XE 
echo dbuser            -  User who can execute common scripts ie.,dbCommonUserId
echo.
echo Examples:
echo         1. upgradeSchema.bat
echo         2. upgradeSchema.bat 602 orcl dbCommonUserId
echo ======================================================================

goto END

:END

endlocal