--
-- Licensed Materials - Property of IBM
-- 5655-FLW (C) Copyright IBM Corporation 2006,2008.
-- All Rights Reserved.
-- US Government Users Restricted Rights- 
-- Use, duplication or disclosure restricted 
-- by GSA ADP Schedule Contract with IBM Corp.
--
-- Scriptfile to upgrade schema for DB2 UDB
-- Process this script in the DB2 command line processor
--
-- Example:
--             db2 connect to BPEDB
--             db2 -tf upgradeSchema602nonp.sql
--


----------------
-- Drop views --
----------------

DROP VIEW PROCESS_TEMPLATE;

DROP VIEW PROCESS_TEMPL_ATTR;

DROP VIEW PROCESS_INSTANCE;

DROP VIEW PROCESS_ATTRIBUTE;

DROP VIEW ACTIVITY;

DROP VIEW ACTIVITY_ATTRIBUTE;

DROP VIEW EVENT;

DROP VIEW WORK_ITEM;

DROP VIEW ACTIVITY_SERVICE;

DROP VIEW QUERY_PROPERTY;

DROP VIEW AUDIT_LOG;

DROP VIEW AUDIT_LOG_B;

DROP VIEW APPLICATION_COMP;

DROP VIEW TASK;

DROP VIEW TASK_TEMPL;

DROP VIEW ESCALATION;

DROP VIEW ESC_TEMPL;

DROP VIEW ESC_TEMPL_DESC;

DROP VIEW TASK_TEMPL_CPROP;

DROP VIEW ESC_TEMPL_CPROP;

DROP VIEW TASK_TEMPL_DESC;

DROP VIEW TASK_CPROP;

DROP VIEW TASK_DESC;

DROP VIEW ESCALATION_CPROP;

DROP VIEW ESCALATION_DESC;

DROP VIEW CUSTOM_PROPERTY;

DROP VIEW DESCRIPTION;

DROP VIEW TASK_AUDIT_LOG;

------------------
-- Drop indexes --
------------------

DROP INDEX AIB_ATID_PI_EH_FE;

DROP INDEX EI_PIID_VTID;

DROP INDEX EI_STATE_VTID;

DROP INDEX WI_ASSOBJ_REASON;

DROP INDEX WI_EVERYBODY;

DROP INDEX WI_ASSOBJ_QIID;

DROP INDEX COS_PIID;

DROP INDEX CP_COORDHOME;

-------------------
-- Create tables --
-------------------

CREATE TABLE PROGRESS_COUNTER_T
(
  PIID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  COUNTER                            BIGINT                            NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PIID )
);

ALTER TABLE PROGRESS_COUNTER_T VOLATILE;

CREATE TABLE RELEVANT_SCOPE_ATASK_T
(
  SIID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  TKIID                              CHAR(16)            FOR BIT DATA  NOT NULL ,
  PIID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( SIID, TKIID )
);

ALTER TABLE RELEVANT_SCOPE_ATASK_T VOLATILE;

CREATE TABLE CONFIG_INFO_T
(
  IDENTIFIER                         VARCHAR(64)                       NOT NULL ,
  DESCRIPTION                        VARCHAR(128)                               ,
  PROPERTY                           INTEGER                                    ,
  COUNTER                            BIGINT                                     ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( IDENTIFIER )
);

ALTER TABLE CONFIG_INFO_T VOLATILE;

CREATE TABLE SYNC_T
(
  IDENTIFIER                         VARCHAR(254)                      NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( IDENTIFIER )
);

CREATE TABLE MV_CTR_T
(
  ID                                 INTEGER                           NOT NULL ,
  SELECT_CLAUSE                      VARCHAR(1024)                     NOT NULL ,
  WHERE_CLAUSE                       VARCHAR(1024)                              ,
  ORDER_CLAUSE                       VARCHAR(512)                               ,
  TBL_SPACE                          VARCHAR(32)                       NOT NULL ,
  UPDATED                            TIMESTAMP                         NOT NULL ,
  UPD_STARTED                        TIMESTAMP                                  ,
  AVG_UPD_TIME                       BIGINT                                     ,
  UPD_INTERVAL                       BIGINT                            NOT NULL ,
  IS_UPDATING                        SMALLINT                          NOT NULL ,
  ACTIVE_MV                          SMALLINT                          NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( ID )
);

CREATE TABLE MV_MODS_T
(
  PKID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  OID                                CHAR(16)            FOR BIT DATA  NOT NULL ,
  MODIFIED                           TIMESTAMP                         NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PKID )
);

ALTER TABLE MV_MODS_T VOLATILE;

CREATE TABLE SAVED_ENGINE_MESSAGE_B_T
(
  PKID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  PIID                               CHAR(16)            FOR BIT DATA           ,
  CREATION_TIME                      TIMESTAMP                         NOT NULL ,
  REASON                             INTEGER                           NOT NULL ,
  ENGINE_MESSAGE_S                   VARCHAR(2000)       FOR BIT DATA           ,
  ENGINE_MESSAGE_L                   BLOB(1073741823)                           ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PKID )
);

ALTER TABLE SAVED_ENGINE_MESSAGE_B_T VOLATILE;

CREATE INDEX SEM_PIID ON SAVED_ENGINE_MESSAGE_B_T
(   
  PIID
);

CREATE INDEX SEM_REAS ON SAVED_ENGINE_MESSAGE_B_T
(   
  REASON
);

CREATE TABLE NAVIGATION_CLEANUP_TIMER_B_T
(
  PKID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  SCHEDULED_TIME                     TIMESTAMP                         NOT NULL ,
  SCHEDULER_ID                       VARCHAR(254)                      NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PKID )
);

ALTER TABLE NAVIGATION_CLEANUP_TIMER_B_T VOLATILE;

CREATE TABLE SCHEDULER_ACTION_T
(
  SCHEDULER_ID                       VARCHAR(254)                      NOT NULL ,
  OID                                CHAR(16)            FOR BIT DATA  NOT NULL ,
  ACTION_OBJECT                      BLOB(3900k)                                ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( SCHEDULER_ID )
);

ALTER TABLE SCHEDULER_ACTION_T VOLATILE;

CREATE INDEX SCEACT_OID ON SCHEDULER_ACTION_T
(   
  OID
);

CREATE TABLE QUERY_TABLE_T
(
  NAME                               VARCHAR(32)                       NOT NULL ,
  KIND                               INTEGER                           NOT NULL ,
  TYPE                               INTEGER                           NOT NULL ,
  VERSION                            INTEGER                           NOT NULL ,
  LAST_UPDATED                       TIMESTAMP                         NOT NULL ,
  PRIMARY_VIEW                       VARCHAR(32)                                ,
  DEFINITION                         BLOB(1073741823)                  NOT NULL ,
  VV_CFG                             BLOB(1073741823)                           ,
  DV_CFG                             BLOB(1073741823)                           ,
  MV_CFG                             BLOB(1073741823)                           ,
  MT_CFG                             BLOB(1073741823)                           ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( NAME )
);

ALTER TABLE QUERY_TABLE_T VOLATILE;

CREATE INDEX QT_KI ON QUERY_TABLE_T
(   
  KIND
);

CREATE INDEX QT_TY ON QUERY_TABLE_T
(   
  TYPE
);

CREATE TABLE QUERY_TABLE_REF_T
(
  REFEREE                            VARCHAR(32)                       NOT NULL ,
  SOURCE                             VARCHAR(32)                       NOT NULL ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( REFEREE, SOURCE )
);

ALTER TABLE QUERY_TABLE_REF_T VOLATILE;

CREATE INDEX QTR_SRC ON QUERY_TABLE_REF_T
(   
  SOURCE
);

CREATE TABLE TASK_HISTORY_T
(
  PKID                               CHAR(16)            FOR BIT DATA  NOT NULL ,
  EVENT                              INTEGER                           NOT NULL ,
  TKIID                              CHAR(16)            FOR BIT DATA  NOT NULL ,
  CONTAINMENT_CONTEXT_ID             CHAR(16)            FOR BIT DATA  NOT NULL ,
  PARENT_TKIID                       CHAR(16)            FOR BIT DATA           ,
  ESIID                              CHAR(16)            FOR BIT DATA           ,
  REASON                             INTEGER                           NOT NULL ,
  EVENT_TIME                         TIMESTAMP                         NOT NULL ,
  NEXT_TIME                          TIMESTAMP                                  ,
  PRINCIPAL                          VARCHAR(128)                      NOT NULL ,
  WORK_ITEM_KIND                     INTEGER                           NOT NULL ,
  FROM_ID                            VARCHAR(128)                               ,
  TO_ID                              VARCHAR(128)                               ,
  VERSION_ID                         SMALLINT                          NOT NULL ,
  PRIMARY KEY ( PKID )
);

ALTER TABLE TASK_HISTORY_T VOLATILE;

CREATE INDEX TH_TKIID ON TASK_HISTORY_T
(   
  TKIID
);

CREATE INDEX TH_CTXID ON TASK_HISTORY_T
(   
  CONTAINMENT_CONTEXT_ID
);
ALTER TABLE PROCESS_TEMPLATE_B_T
 ADD 
    EXTENDED_AUTO_DELETE               INTEGER                           DEFAULT 2 NOT NULL ;
ALTER TABLE PROCESS_TEMPLATE_B_T
 ADD 
    CONTINUE_ON_ERROR                  SMALLINT                          DEFAULT 1 NOT NULL ;
ALTER TABLE PROCESS_TEMPLATE_B_T
 ADD 
    IGNORE_MISSING_DATA                INTEGER                           DEFAULT 0 NOT NULL ;

ALTER TABLE SCOPE_TEMPLATE_B_T
 ADD 
    A_TKTID                            CHAR(16)            FOR BIT DATA ;
ALTER TABLE SCOPE_TEMPLATE_B_T
 ADD 
    IS_IMPLICIT                        SMALLINT                          DEFAULT 0 NOT NULL ;

ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXIT_CONDITION                     INTEGER                           DEFAULT 1 NOT NULL ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXIT_CONDITION_NAME                VARCHAR(254)                     ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXIT_CONDITION_EXECUTE_AT          INTEGER                           DEFAULT 3 NOT NULL ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    SOURCES_TYPE                       INTEGER                           DEFAULT 0 NOT NULL ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    TARGETS_TYPE                       INTEGER                           DEFAULT 0 NOT NULL ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    PARENT_ATID                        CHAR(16)            FOR BIT DATA ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    HAS_CROSSING_LINK                  SMALLINT                         ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXIT_EXPRESSION                    CLOB(3900K)                      ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXPRESSION_MAP                     BLOB(3900K)                      ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    EXIT_EXPRESSION_MAP                BLOB(3900K)                      ;
ALTER TABLE ACTIVITY_TEMPLATE_B_T
 ADD 
    GENERATED_BY                       VARCHAR(220)                     ;

ALTER TABLE ALARM_TEMPLATE_B_T
 ADD 
    EXPRESSION_MAP                     BLOB(3900K)                      ;
ALTER TABLE ALARM_TEMPLATE_B_T
 ADD 
    REPEAT_EXPRESSION_MAP              BLOB(3900K)                      ;

ALTER TABLE FAULT_HANDLER_TEMPLATE_B_T
 ADD 
    FAULT_LINK_SOURCE_ATID             CHAR(16)            FOR BIT DATA ;
ALTER TABLE FAULT_HANDLER_TEMPLATE_B_T
 ADD 
    FAULT_LINK_TARGET_ATID             CHAR(16)            FOR BIT DATA ;

ALTER TABLE LINK_TEMPLATE_B_T
 ADD 
    EXPRESSION_MAP                     BLOB(3900K)                      ;
ALTER TABLE LINK_TEMPLATE_B_T
 ADD 
    GENERATED_BY                       VARCHAR(220)                     ;

ALTER TABLE VARIABLE_TEMPLATE_B_T
 ADD 
    GENERATED_BY                       VARCHAR(220)                     ;

ALTER TABLE ASSIGN_TEMPLATE_B_T
 ADD 
    FROM_EXPRESSION_MAP                BLOB(3900K)                      ;
ALTER TABLE ASSIGN_TEMPLATE_B_T
 ADD 
    TO_EXPRESSION_MAP                  BLOB(3900K)                      ;

ALTER TABLE EHALARM_TEMPLATE_B_T
 ADD 
    EXPRESSION_MAP                     BLOB(3900K)                      ;
ALTER TABLE EHALARM_TEMPLATE_B_T
 ADD 
    REPEAT_EXPRESSION_MAP              BLOB(3900K)                      ;

ALTER TABLE FOR_EACH_TEMPLATE_B_T
 ADD 
    START_COUNTER_EXPRESSION_MAP       BLOB(3900K)                      ;
ALTER TABLE FOR_EACH_TEMPLATE_B_T
 ADD 
    FINAL_COUNTER_EXPRESSION_MAP       BLOB(3900K)                      ;
ALTER TABLE FOR_EACH_TEMPLATE_B_T
 ADD 
    COMPLETION_COND_EXPRESSION_MAP     BLOB(3900K)                      ;

ALTER TABLE STAFF_QUERY_TEMPLATE_T
 ADD 
    STAFF_VERB                         BLOB(3900K)                      ;

ALTER TABLE PROCESS_INSTANCE_B_T
 ADD 
    SCHEDULER_TASK_ID                  VARCHAR(254)                     ;
ALTER TABLE PROCESS_INSTANCE_B_T
 ADD 
    RESUMES                            TIMESTAMP                        ;
ALTER TABLE PROCESS_INSTANCE_B_T
 ADD 
    PENDING_SKIP_REQUEST               SMALLINT                          DEFAULT 0 NOT NULL ;
ALTER TABLE PROCESS_INSTANCE_B_T
 ADD 
    UNHANDLED_EXCEPTION                BLOB(1073741823)                 ;

ALTER TABLE SCOPE_INSTANCE_B_T
 ADD 
    A_TKIID                            CHAR(16)            FOR BIT DATA ;

ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    INVOKED_INSTANCE_ID                CHAR(16)            FOR BIT DATA ;
ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    INVOKED_INSTANCE_TYPE              INTEGER                           DEFAULT 0 NOT NULL ;
ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    STOP_REASON                        INTEGER                           DEFAULT 1 NOT NULL ;
ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    PREVIOUS_STATE                     INTEGER                          ;
ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    SKIP_REQUESTED                     SMALLINT                          DEFAULT 0 NOT NULL ;
ALTER TABLE ACTIVITY_INSTANCE_B_T
 ADD 
    JUMP_TARGET_ATID                   CHAR(16)            FOR BIT DATA ;

ALTER TABLE SCOPED_VARIABLE_INSTANCE_B_T
 ADD 
    DATA_IL                            VARCHAR(3000)       FOR BIT DATA ;

ALTER TABLE PROCESS_INSTANCE_ATTRIBUTE_T
 ADD 
    DATA_TYPE                          VARCHAR(254)                     ;
ALTER TABLE PROCESS_INSTANCE_ATTRIBUTE_T
 ADD 
    DATA                               BLOB(1073741823)                 ;

ALTER TABLE PROCESS_CONTEXT_T
 ADD 
    STARTER_EXPIRES                    TIMESTAMP                        ;

ALTER TABLE ACTIVITY_INSTANCE_ATTR_B_T
 ADD 
    DATA_TYPE                          VARCHAR(254)                     ;
ALTER TABLE ACTIVITY_INSTANCE_ATTR_B_T
 ADD 
    DATA                               BLOB(1073741823)                 ;

ALTER TABLE WORK_ITEM_T
 ADD 
    PARENT_WIID                        CHAR(16)            FOR BIT DATA ;
ALTER TABLE WORK_ITEM_T
 ADD 
    AUTH_INFO                          INTEGER                           DEFAULT 0 NOT NULL ;

ALTER TABLE WI_ASSOC_OID_T
 ADD 
    MIGRATED                           SMALLINT                          DEFAULT 0 NOT NULL ;

ALTER TABLE APPLICATION_COMPONENT_T
 ADD 
    SUBSTITUTION_POLICY                INTEGER                           DEFAULT 0 NOT NULL ;

ALTER TABLE TASK_TEMPLATE_T
 ADD 
    AUTO_DELETE_MODE                   INTEGER                           DEFAULT 2 NOT NULL ;
ALTER TABLE TASK_TEMPLATE_T
 ADD 
    AUTONOMY                           INTEGER                           DEFAULT 1 NOT NULL ;
ALTER TABLE TASK_TEMPLATE_T
 ADD 
    PRIORITY_DEFINITION                VARCHAR(254)                     ;
ALTER TABLE TASK_TEMPLATE_T
 ADD 
    SUBSTITUTION_POLICY                INTEGER                           DEFAULT 0 NOT NULL ;

ALTER TABLE ESCALATION_INSTANCE_T
 ADD 
    ESCALATION_TIME                    TIMESTAMP                        ;

ALTER TABLE TASK_INSTANCE_T
 ADD 
    AUTO_DELETE_MODE                   INTEGER                           DEFAULT 2 NOT NULL ;
ALTER TABLE TASK_INSTANCE_T
 ADD 
    IS_CHILD                           SMALLINT                          DEFAULT 0 NOT NULL ;
ALTER TABLE TASK_INSTANCE_T
 ADD 
    HAS_ESCALATIONS                    SMALLINT                         ;
ALTER TABLE TASK_INSTANCE_T
 ADD 
    RESUMES                            TIMESTAMP                        ;
ALTER TABLE TASK_INSTANCE_T
 ADD 
    SUBSTITUTION_POLICY                INTEGER                           DEFAULT 0 NOT NULL ;
ALTER TABLE TASK_INSTANCE_T
 ADD 
    DELETION_TIME                      TIMESTAMP                        ;

ALTER TABLE WI_ASSOC_OID_T VOLATILE;


CREATE INDEX AIB_PI_ATID_EH_FE ON ACTIVITY_INSTANCE_B_T
(   
  PIID, ATID, EHIID, ENCLOSING_FEIID
);

CREATE INDEX EI_PIID_PRIM ON EVENT_INSTANCE_B_T
(   
  PIID, IS_PRIMARY_EVENT_INSTANCE, STATE
);

CREATE INDEX EI_PIID_VTID ON EVENT_INSTANCE_B_T
(   
  PIID, VTID, IS_PRIMARY_EVENT_INSTANCE
);

CREATE INDEX EI_STATE_VTID ON EVENT_INSTANCE_B_T
(   
  VTID, STATE, EIID, AIID, PIID
);

CREATE INDEX PIA_DATY ON PROCESS_INSTANCE_ATTRIBUTE_T
(   
  DATA_TYPE
);

CREATE INDEX AIA_DATY ON ACTIVITY_INSTANCE_ATTR_B_T
(   
  DATA_TYPE
);

CREATE INDEX AWI_AIID ON AWAITED_INVOCATION_T
(   
  AIID
);

CREATE INDEX WI_ASSOBJ_REASON ON WORK_ITEM_T
(   
  ASSOCIATED_OID, ASSOCIATED_OBJECT_TYPE, REASON, PARENT_WIID
);

CREATE INDEX WI_AUTH_L ON WORK_ITEM_T
(   
  EVERYBODY, GROUP_NAME, OWNER_ID, QIID
);

CREATE INDEX WI_AUTH_U ON WORK_ITEM_T
(   
  AUTH_INFO, QIID
);

CREATE INDEX WI_AUTH_O ON WORK_ITEM_T
(   
  AUTH_INFO, OWNER_ID DESC
);

CREATE INDEX WI_AUTH_G ON WORK_ITEM_T
(   
  AUTH_INFO, GROUP_NAME
);

CREATE INDEX WI_AUTH_E ON WORK_ITEM_T
(   
  AUTH_INFO, EVERYBODY
);

CREATE INDEX WI_AUTH_R ON WORK_ITEM_T
(   
  AUTH_INFO, REASON DESC
);

CREATE INDEX WI_QI_OID_RS_OWN ON WORK_ITEM_T
(   
  QIID, OBJECT_ID, REASON, OWNER_ID
);

CREATE INDEX WI_QI_OID_OWN ON WORK_ITEM_T
(   
  QIID, OBJECT_ID, OWNER_ID
);

CREATE INDEX WI_OT_OID_RS ON WORK_ITEM_T
(   
  OBJECT_TYPE, OBJECT_ID, REASON
);

CREATE INDEX WI_WI_QI ON WORK_ITEM_T
(   
  WIID, QIID
);

CREATE INDEX WI_PARENT_WIID ON WORK_ITEM_T
(   
  PARENT_WIID
);

CREATE INDEX WI_AUTH_GR_O_E ON WORK_ITEM_T
(   
  AUTH_INFO, GROUP_NAME, OWNER_ID, EVERYBODY
);

CREATE INDEX WIASS_WI ON WI_ASSOC_OID_T
(   
  WIID
);

CREATE INDEX WIASS_MIG ON WI_ASSOC_OID_T
(   
  MIGRATED, WIID, OBJECT_ID, OBJECT_TYPE, ASSOCIATED_OID, REASON
);

CREATE INDEX RUT_QIID ON RETRIEVED_USER_T
(   
  QIID
);

CREATE INDEX RUT_OWN_QIDESC ON RETRIEVED_USER_T
(   
  OWNER_ID, QIID DESC
);

CREATE INDEX TT_KND_INLN_VAL ON TASK_TEMPLATE_T
(   
  KIND, IS_INLINE, VALID_FROM
);

CREATE INDEX TTLD_TT_LOC ON TASK_TEMPL_LDESC_T
(   
  TKTID, LOCALE DESC
);

CREATE INDEX ESI_EST_ESI ON ESCALATION_INSTANCE_T
(   
  ESTID, ESIID
);

CREATE INDEX TI_TI_KND_ST ON TASK_INSTANCE_T
(   
  TKIID, KIND, STATE
);

CREATE INDEX TI_ST_KND_TI_NAME ON TASK_INSTANCE_T
(   
  STATE, KIND, TKIID, NAME
);

CREATE INDEX TI_TT_KND ON TASK_INSTANCE_T
(   
  TKTID, KIND
);

-- View: ProcessTemplate
CREATE VIEW PROCESS_TEMPLATE(PTID, NAME, DISPLAY_NAME, VALID_FROM, TARGET_NAMESPACE, APPLICATION_NAME, VERSION, CREATED, STATE, EXECUTION_MODE, DESCRIPTION, CAN_RUN_SYNC, CAN_RUN_INTERRUP, COMP_SPHERE, CONTINUE_ON_ERROR ) AS
 SELECT PROCESS_TEMPLATE_B_T.PTID, PROCESS_TEMPLATE_B_T.NAME, PROCESS_TEMPLATE_B_T.DISPLAY_NAME, PROCESS_TEMPLATE_B_T.VALID_FROM, PROCESS_TEMPLATE_B_T.TARGET_NAMESPACE, PROCESS_TEMPLATE_B_T.APPLICATION_NAME, PROCESS_TEMPLATE_B_T.VERSION, PROCESS_TEMPLATE_B_T.CREATED, PROCESS_TEMPLATE_B_T.STATE, PROCESS_TEMPLATE_B_T.EXECUTION_MODE, PROCESS_TEMPLATE_B_T.DESCRIPTION, PROCESS_TEMPLATE_B_T.CAN_CALL, PROCESS_TEMPLATE_B_T.CAN_INITIATE, PROCESS_TEMPLATE_B_T.COMPENSATION_SPHERE, PROCESS_TEMPLATE_B_T.CONTINUE_ON_ERROR
 FROM PROCESS_TEMPLATE_B_T;

-- View: ProcessTemplAttr
CREATE VIEW PROCESS_TEMPL_ATTR(PTID, NAME, VALUE ) AS
 SELECT PROCESS_TEMPLATE_ATTRIBUTE_B_T.PTID, PROCESS_TEMPLATE_ATTRIBUTE_B_T.ATTR_KEY, PROCESS_TEMPLATE_ATTRIBUTE_B_T.VALUE
 FROM PROCESS_TEMPLATE_ATTRIBUTE_B_T;

-- View: ProcessInstance
CREATE VIEW PROCESS_INSTANCE(PTID, PIID, NAME, STATE, CREATED, STARTED, COMPLETED, PARENT_NAME, TOP_LEVEL_NAME, PARENT_PIID, TOP_LEVEL_PIID, STARTER, DESCRIPTION, TEMPLATE_NAME, TEMPLATE_DESCR, RESUMES, CONTINUE_ON_ERROR ) AS
 SELECT PROCESS_INSTANCE_B_T.PTID, PROCESS_INSTANCE_B_T.PIID, PROCESS_INSTANCE_B_T.NAME, PROCESS_INSTANCE_B_T.STATE, PROCESS_INSTANCE_B_T.CREATED, PROCESS_INSTANCE_B_T.STARTED, PROCESS_INSTANCE_B_T.COMPLETED, PROCESS_INSTANCE_B_T.PARENT_NAME, PROCESS_INSTANCE_B_T.TOP_LEVEL_NAME, PROCESS_INSTANCE_B_T.PARENT_PIID, PROCESS_INSTANCE_B_T.TOP_LEVEL_PIID, PROCESS_INSTANCE_B_T.STARTER, PROCESS_INSTANCE_B_T.DESCRIPTION, PROCESS_TEMPLATE_B_T.NAME, PROCESS_TEMPLATE_B_T.DESCRIPTION, PROCESS_INSTANCE_B_T.RESUMES, PROCESS_TEMPLATE_B_T.CONTINUE_ON_ERROR
 FROM PROCESS_INSTANCE_B_T, PROCESS_TEMPLATE_B_T
 WHERE PROCESS_INSTANCE_B_T.PTID = PROCESS_TEMPLATE_B_T.PTID;

-- View: ProcessAttribute
CREATE VIEW PROCESS_ATTRIBUTE(PIID, NAME, VALUE, DATA_TYPE ) AS
 SELECT PROCESS_INSTANCE_ATTRIBUTE_T.PIID, PROCESS_INSTANCE_ATTRIBUTE_T.ATTR_KEY, PROCESS_INSTANCE_ATTRIBUTE_T.VALUE, PROCESS_INSTANCE_ATTRIBUTE_T.DATA_TYPE
 FROM PROCESS_INSTANCE_ATTRIBUTE_T;

-- View: Activity
CREATE VIEW ACTIVITY(PIID, AIID, PTID, ATID, SIID, STID, EHIID, ENCLOSING_FEIID, KIND, COMPLETED, ACTIVATED, FIRST_ACTIVATED, STARTED, STATE, STOP_REASON, OWNER, DESCRIPTION, EXPIRES, TEMPLATE_NAME, TEMPLATE_DESCR, BUSINESS_RELEVANCE, SKIP_REQUESTED, CONTINUE_ON_ERROR, INVOKED_INST_ID, INVOKED_INST_TYPE ) AS
 SELECT ACTIVITY_INSTANCE_B_T.PIID, ACTIVITY_INSTANCE_B_T.AIID, ACTIVITY_INSTANCE_B_T.PTID, ACTIVITY_INSTANCE_B_T.ATID, ACTIVITY_INSTANCE_B_T.SIID, ACTIVITY_TEMPLATE_B_T.PARENT_STID, ACTIVITY_INSTANCE_B_T.EHIID, ACTIVITY_INSTANCE_B_T.ENCLOSING_FEIID, ACTIVITY_TEMPLATE_B_T.KIND, ACTIVITY_INSTANCE_B_T.FINISHED, ACTIVITY_INSTANCE_B_T.ACTIVATED, ACTIVITY_INSTANCE_B_T.FIRST_ACTIVATED, ACTIVITY_INSTANCE_B_T.STARTED, ACTIVITY_INSTANCE_B_T.STATE, ACTIVITY_INSTANCE_B_T.STOP_REASON, ACTIVITY_INSTANCE_B_T.OWNER, ACTIVITY_INSTANCE_B_T.DESCRIPTION, ACTIVITY_INSTANCE_B_T.EXPIRES, ACTIVITY_TEMPLATE_B_T.NAME, ACTIVITY_TEMPLATE_B_T.DESCRIPTION, ACTIVITY_TEMPLATE_B_T.BUSINESS_RELEVANCE, ACTIVITY_INSTANCE_B_T.SKIP_REQUESTED, ACTIVITY_INSTANCE_B_T.CONTINUE_ON_ERROR, ACTIVITY_INSTANCE_B_T.INVOKED_INSTANCE_ID, ACTIVITY_INSTANCE_B_T.INVOKED_INSTANCE_TYPE
 FROM ACTIVITY_INSTANCE_B_T, ACTIVITY_TEMPLATE_B_T
 WHERE ACTIVITY_INSTANCE_B_T.ATID = ACTIVITY_TEMPLATE_B_T.ATID;

-- View: ActivityAttribute
CREATE VIEW ACTIVITY_ATTRIBUTE(AIID, NAME, VALUE, DATA_TYPE ) AS
 SELECT ACTIVITY_INSTANCE_ATTR_B_T.AIID, ACTIVITY_INSTANCE_ATTR_B_T.ATTR_KEY, ACTIVITY_INSTANCE_ATTR_B_T.ATTR_VALUE, ACTIVITY_INSTANCE_ATTR_B_T.DATA_TYPE
 FROM ACTIVITY_INSTANCE_ATTR_B_T;

-- View: Event
CREATE VIEW EVENT(EIID, AIID, PIID, EHTID, SIID, NAME ) AS
 SELECT EVENT_INSTANCE_B_T.EIID, EVENT_INSTANCE_B_T.AIID, EVENT_INSTANCE_B_T.PIID, EVENT_INSTANCE_B_T.EHTID, EVENT_INSTANCE_B_T.SIID, SERVICE_TEMPLATE_B_T.NAME
 FROM EVENT_INSTANCE_B_T, SERVICE_TEMPLATE_B_T
 WHERE EVENT_INSTANCE_B_T.STATE = 2 AND EVENT_INSTANCE_B_T.VTID = SERVICE_TEMPLATE_B_T.VTID;

-- View: WorkItem
CREATE VIEW WORK_ITEM(WIID, OWNER_ID, GROUP_NAME, EVERYBODY, OBJECT_TYPE, OBJECT_ID, ASSOC_OBJECT_TYPE, ASSOC_OID, REASON, CREATION_TIME, QIID, KIND ) AS
 SELECT WORK_ITEM_T.WIID, WORK_ITEM_T.OWNER_ID, WORK_ITEM_T.GROUP_NAME, WORK_ITEM_T.EVERYBODY, WORK_ITEM_T.OBJECT_TYPE, WORK_ITEM_T.OBJECT_ID, WORK_ITEM_T.ASSOCIATED_OBJECT_TYPE, WORK_ITEM_T.ASSOCIATED_OID, WORK_ITEM_T.REASON, WORK_ITEM_T.CREATION_TIME, WORK_ITEM_T.QIID, WORK_ITEM_T.KIND
 FROM WORK_ITEM_T
 WHERE WORK_ITEM_T.QIID IS NULL OR WORK_ITEM_T.EVERYBODY = 1
 UNION ALL SELECT WORK_ITEM_T.WIID, WORK_ITEM_T.OWNER_ID, WORK_ITEM_T.GROUP_NAME, WORK_ITEM_T.EVERYBODY, WI_ASSOC_OID_T.OBJECT_TYPE, WI_ASSOC_OID_T.OBJECT_ID, WORK_ITEM_T.ASSOCIATED_OBJECT_TYPE, WORK_ITEM_T.ASSOCIATED_OID, WI_ASSOC_OID_T.REASON, WORK_ITEM_T.CREATION_TIME, WORK_ITEM_T.QIID, WORK_ITEM_T.KIND
 FROM WORK_ITEM_T, WI_ASSOC_OID_T
 WHERE WORK_ITEM_T.WIID = WI_ASSOC_OID_T.WIID AND (WORK_ITEM_T.QIID IS NULL OR WORK_ITEM_T.EVERYBODY = 1)
 UNION ALL SELECT WORK_ITEM_T.WIID, RETRIEVED_USER_T.OWNER_ID, WORK_ITEM_T.GROUP_NAME, WORK_ITEM_T.EVERYBODY, WORK_ITEM_T.OBJECT_TYPE, WORK_ITEM_T.OBJECT_ID, WORK_ITEM_T.ASSOCIATED_OBJECT_TYPE, WORK_ITEM_T.ASSOCIATED_OID, WORK_ITEM_T.REASON, WORK_ITEM_T.CREATION_TIME, WORK_ITEM_T.QIID, WORK_ITEM_T.KIND
 FROM WORK_ITEM_T, RETRIEVED_USER_T
 WHERE WORK_ITEM_T.QIID = RETRIEVED_USER_T.QIID
 UNION ALL SELECT WORK_ITEM_T.WIID, RETRIEVED_USER_T.OWNER_ID, WORK_ITEM_T.GROUP_NAME, WORK_ITEM_T.EVERYBODY, WI_ASSOC_OID_T.OBJECT_TYPE, WI_ASSOC_OID_T.OBJECT_ID, WORK_ITEM_T.ASSOCIATED_OBJECT_TYPE, WORK_ITEM_T.ASSOCIATED_OID, WI_ASSOC_OID_T.REASON, WORK_ITEM_T.CREATION_TIME, WORK_ITEM_T.QIID, WORK_ITEM_T.KIND
 FROM WORK_ITEM_T, RETRIEVED_USER_T, WI_ASSOC_OID_T
 WHERE WORK_ITEM_T.QIID = RETRIEVED_USER_T.QIID AND WORK_ITEM_T.WIID = WI_ASSOC_OID_T.WIID;

-- View: ActivityService
CREATE VIEW ACTIVITY_SERVICE(EIID, AIID, PIID, VTID, PORT_TYPE, NAME_SPACE_URI, OPERATION ) AS
 SELECT EVENT_INSTANCE_B_T.EIID, EVENT_INSTANCE_B_T.AIID, EVENT_INSTANCE_B_T.PIID, EVENT_INSTANCE_B_T.VTID, SERVICE_TEMPLATE_B_T.PORT_TYPE_NAME,URI_TEMPLATE_B_T.URI, SERVICE_TEMPLATE_B_T.OPERATION_NAME
 FROM EVENT_INSTANCE_B_T, SERVICE_TEMPLATE_B_T, URI_TEMPLATE_B_T
 WHERE EVENT_INSTANCE_B_T.STATE = 2 AND EVENT_INSTANCE_B_T.VTID = SERVICE_TEMPLATE_B_T.VTID AND SERVICE_TEMPLATE_B_T.PORT_TYPE_UTID = URI_TEMPLATE_B_T.UTID;

-- View: QueryProperty
CREATE VIEW QUERY_PROPERTY(PIID, VARIABLE_NAME, NAME, NAMESPACE, GENERIC_VALUE, STRING_VALUE, NUMBER_VALUE, DECIMAL_VALUE, TIMESTAMP_VALUE ) AS
 SELECT QUERYABLE_VARIABLE_INSTANCE_T.PIID, QUERYABLE_VARIABLE_INSTANCE_T.VARIABLE_NAME, QUERYABLE_VARIABLE_INSTANCE_T.PROPERTY_NAME, QUERYABLE_VARIABLE_INSTANCE_T.PROPERTY_NAMESPACE, QUERYABLE_VARIABLE_INSTANCE_T.GENERIC_VALUE, QUERYABLE_VARIABLE_INSTANCE_T.STRING_VALUE, QUERYABLE_VARIABLE_INSTANCE_T.NUMBER_VALUE, QUERYABLE_VARIABLE_INSTANCE_T.DECIMAL_VALUE, QUERYABLE_VARIABLE_INSTANCE_T.TIMESTAMP_VALUE
 FROM QUERYABLE_VARIABLE_INSTANCE_T;

-- View: QueryPropTempl
CREATE VIEW QUERY_PROP_TEMPL(PTID, NAME, PROPERTY_NAME, URI, QUERY_TYPE, JAVA_TYPE ) AS
 SELECT QUERYABLE_VARIABLE_TEMPLATE_T.PTID, VARIABLE_TEMPLATE_B_T.NAME, PROPERTY_ALIAS_TEMPLATE_B_T.PROPERTY_NAME, URI_TEMPLATE_B_T.URI, QUERYABLE_VARIABLE_TEMPLATE_T.QUERY_TYPE, PROPERTY_ALIAS_TEMPLATE_B_T.JAVA_TYPE
 FROM QUERYABLE_VARIABLE_TEMPLATE_T, VARIABLE_TEMPLATE_B_T, PROPERTY_ALIAS_TEMPLATE_B_T, URI_TEMPLATE_B_T
 WHERE QUERYABLE_VARIABLE_TEMPLATE_T.CTID = VARIABLE_TEMPLATE_B_T.CTID AND QUERYABLE_VARIABLE_TEMPLATE_T.PAID = PROPERTY_ALIAS_TEMPLATE_B_T.PAID AND PROPERTY_ALIAS_TEMPLATE_B_T.PROPERTY_UTID = URI_TEMPLATE_B_T.UTID;

-- View: AuditLog
CREATE VIEW AUDIT_LOG(ALID, EVENT_TIME, EVENT_TIME_UTC, AUDIT_EVENT, PTID, PIID, AIID, SIID, SCOPE_NAME, PROCESS_TEMPL_NAME, PROCESS_INST_NAME, TOP_LEVEL_PI_NAME, TOP_LEVEL_PIID, PARENT_PI_NAME, PARENT_PIID, VALID_FROM, VALID_FROM_UTC, ACTIVITY_NAME, ACTIVITY_KIND, ACTIVITY_STATE, CONTROL_LINK_NAME, IMPL_NAME, PRINCIPAL, TERMINAL_NAME, VARIABLE_DATA, EXCEPTION_TEXT, DESCRIPTION, CORR_SET_INFO, USER_NAME, ATID, ADDITIONAL_INFO, OBJECT_META_TYPE ) AS
 SELECT AUDIT_LOG_T.ALID,AUDIT_LOG_T.EVENT_TIME,AUDIT_LOG_T.EVENT_TIME_UTC,AUDIT_LOG_T.AUDIT_EVENT,AUDIT_LOG_T.PTID,AUDIT_LOG_T.PIID,AUDIT_LOG_T.AIID,AUDIT_LOG_T.SIID,AUDIT_LOG_T.VARIABLE_NAME,AUDIT_LOG_T.PROCESS_TEMPL_NAME, AUDIT_LOG_T.PROCESS_INST_NAME,AUDIT_LOG_T.TOP_LEVEL_PI_NAME,AUDIT_LOG_T.TOP_LEVEL_PIID,AUDIT_LOG_T.PARENT_PI_NAME,AUDIT_LOG_T.PARENT_PIID,AUDIT_LOG_T.VALID_FROM,AUDIT_LOG_T.VALID_FROM_UTC, AUDIT_LOG_T.ACTIVITY_NAME,AUDIT_LOG_T.ACTIVITY_KIND,AUDIT_LOG_T.ACTIVITY_STATE,AUDIT_LOG_T.CONTROL_LINK_NAME,AUDIT_LOG_T.IMPL_NAME,AUDIT_LOG_T.PRINCIPAL, AUDIT_LOG_T.TERMINAL_NAME,AUDIT_LOG_T.VARIABLE_DATA,AUDIT_LOG_T.EXCEPTION_TEXT,AUDIT_LOG_T.DESCRIPTION,AUDIT_LOG_T.CORR_SET_INFO,AUDIT_LOG_T.USER_NAME, AUDIT_LOG_T.ATID, AUDIT_LOG_T.ADDITIONAL_INFO, AUDIT_LOG_T.OBJECT_META_TYPE
 FROM AUDIT_LOG_T;

-- View: AuditLogB
CREATE VIEW AUDIT_LOG_B(ALID, EVENT_TIME, EVENT_TIME_UTC, AUDIT_EVENT, PTID, PIID, AIID, SIID, VARIABLE_NAME, PROCESS_TEMPL_NAME, TOP_LEVEL_PIID, PARENT_PIID, VALID_FROM, VALID_FROM_UTC, ATID, ACTIVITY_NAME, ACTIVITY_KIND, ACTIVITY_STATE, CONTROL_LINK_NAME, PRINCIPAL, VARIABLE_DATA, EXCEPTION_TEXT, DESCRIPTION, CORR_SET_INFO, USER_NAME, ADDITIONAL_INFO ) AS
 SELECT AUDIT_LOG_T.ALID,AUDIT_LOG_T.EVENT_TIME,AUDIT_LOG_T.EVENT_TIME_UTC,AUDIT_LOG_T.AUDIT_EVENT,AUDIT_LOG_T.PTID,AUDIT_LOG_T.PIID,AUDIT_LOG_T.AIID,AUDIT_LOG_T.SIID,AUDIT_LOG_T.VARIABLE_NAME,AUDIT_LOG_T.PROCESS_TEMPL_NAME, AUDIT_LOG_T.TOP_LEVEL_PIID,AUDIT_LOG_T.PARENT_PIID,AUDIT_LOG_T.VALID_FROM,AUDIT_LOG_T.VALID_FROM_UTC,AUDIT_LOG_T.ATID, AUDIT_LOG_T.ACTIVITY_NAME,AUDIT_LOG_T.ACTIVITY_KIND,AUDIT_LOG_T.ACTIVITY_STATE,AUDIT_LOG_T.CONTROL_LINK_NAME,AUDIT_LOG_T.PRINCIPAL, AUDIT_LOG_T.VARIABLE_DATA,AUDIT_LOG_T.EXCEPTION_TEXT,AUDIT_LOG_T.DESCRIPTION,AUDIT_LOG_T.CORR_SET_INFO,AUDIT_LOG_T.USER_NAME, AUDIT_LOG_T.ADDITIONAL_INFO
 FROM AUDIT_LOG_T
 WHERE AUDIT_LOG_T.OBJECT_META_TYPE = 1;

-- View: ApplicationComp
CREATE VIEW APPLICATION_COMP(ACOID, BUSINESS_RELEVANCE, NAME, SUPPORT_AUTOCLAIM, SUPPORT_CLAIM_SUSP, SUPPORT_DELEGATION, SUPPORT_SUB_TASK, SUPPORT_FOLLOW_ON ) AS
 SELECT APPLICATION_COMPONENT_T.ACOID, APPLICATION_COMPONENT_T.BUSINESS_RELEVANCE, APPLICATION_COMPONENT_T.NAME, APPLICATION_COMPONENT_T.SUPPORTS_AUTO_CLAIM, APPLICATION_COMPONENT_T.SUPPORTS_CLAIM_SUSPENDED, APPLICATION_COMPONENT_T.SUPPORTS_DELEGATION, APPLICATION_COMPONENT_T.SUPPORTS_SUB_TASK, APPLICATION_COMPONENT_T.SUPPORTS_FOLLOW_ON_TASK
 FROM APPLICATION_COMPONENT_T;

-- View: Task
CREATE VIEW TASK(TKIID, ACTIVATED, APPLIC_DEFAULTS_ID, APPLIC_NAME, BUSINESS_RELEVANCE, COMPLETED, CONTAINMENT_CTX_ID, CTX_AUTHORIZATION, DUE, EXPIRES, FIRST_ACTIVATED, FOLLOW_ON_TKIID, IS_AD_HOC, IS_ESCALATED, IS_INLINE, IS_WAIT_FOR_SUB_TK, KIND, LAST_MODIFIED, LAST_STATE_CHANGE, NAME, NAME_SPACE, ORIGINATOR, OWNER, PARENT_CONTEXT_ID, PRIORITY, STARTED, STARTER, STATE, SUPPORT_AUTOCLAIM, SUPPORT_CLAIM_SUSP, SUPPORT_DELEGATION, SUPPORT_SUB_TASK, SUPPORT_FOLLOW_ON, HIERARCHY_POSITION, IS_CHILD, SUSPENDED, TKTID, TOP_TKIID, TYPE, RESUMES ) AS
 SELECT TASK_INSTANCE_T.TKIID, TASK_INSTANCE_T.ACTIVATION_TIME, TASK_INSTANCE_T.APPLICATION_DEFAULTS_ID, TASK_INSTANCE_T.APPLICATION_NAME, TASK_INSTANCE_T.BUSINESS_RELEVANCE, TASK_INSTANCE_T.COMPLETION_TIME, TASK_INSTANCE_T.CONTAINMENT_CONTEXT_ID, TASK_INSTANCE_T.CONTEXT_AUTHORIZATION, TASK_INSTANCE_T.DUE_TIME, TASK_INSTANCE_T.EXPIRATION_TIME, TASK_INSTANCE_T.FIRST_ACTIVATION_TIME, TASK_INSTANCE_T.FOLLOW_ON_TKIID, TASK_INSTANCE_T.IS_AD_HOC, TASK_INSTANCE_T.IS_ESCALATED, TASK_INSTANCE_T.IS_INLINE, TASK_INSTANCE_T.IS_WAITING_FOR_SUBTASK, TASK_INSTANCE_T.KIND, TASK_INSTANCE_T.LAST_MODIFICATION_TIME, TASK_INSTANCE_T.LAST_STATE_CHANGE_TIME, TASK_INSTANCE_T.NAME, TASK_INSTANCE_T.NAMESPACE, TASK_INSTANCE_T.ORIGINATOR, TASK_INSTANCE_T.OWNER, TASK_INSTANCE_T.PARENT_CONTEXT_ID, TASK_INSTANCE_T.PRIORITY, TASK_INSTANCE_T.START_TIME, TASK_INSTANCE_T.STARTER, TASK_INSTANCE_T.STATE, TASK_INSTANCE_T.SUPPORTS_AUTO_CLAIM, TASK_INSTANCE_T.SUPPORTS_CLAIM_SUSPENDED, TASK_INSTANCE_T.SUPPORTS_DELEGATION, TASK_INSTANCE_T.SUPPORTS_SUB_TASK, TASK_INSTANCE_T.SUPPORTS_FOLLOW_ON_TASK, TASK_INSTANCE_T.HIERARCHY_POSITION, TASK_INSTANCE_T.IS_CHILD, TASK_INSTANCE_T.IS_SUSPENDED, TASK_INSTANCE_T.TKTID, TASK_INSTANCE_T.TOP_TKIID, TASK_INSTANCE_T.TYPE, TASK_INSTANCE_T.RESUMES
 FROM TASK_INSTANCE_T;

-- View: TaskTempl
CREATE VIEW TASK_TEMPL(TKTID, APPLIC_DEFAULTS_ID, APPLIC_NAME, BUSINESS_RELEVANCE, CONTAINMENT_CTX_ID, CTX_AUTHORIZATION, DEFINITION_NAME, DEFINITION_NS, IS_AD_HOC, IS_INLINE, KIND, NAME, NAMESPACE, PRIORITY, STATE, SUPPORT_AUTOCLAIM, SUPPORT_CLAIM_SUSP, SUPPORT_DELEGATION, SUPPORT_SUB_TASK, SUPPORT_FOLLOW_ON, TYPE, VALID_FROM, AUTONOMY ) AS
 SELECT TASK_TEMPLATE_T.TKTID, TASK_TEMPLATE_T.APPLICATION_DEFAULTS_ID, TASK_TEMPLATE_T.APPLICATION_NAME, TASK_TEMPLATE_T.BUSINESS_RELEVANCE, TASK_TEMPLATE_T.CONTAINMENT_CONTEXT_ID, TASK_TEMPLATE_T.CONTEXT_AUTHORIZATION, TASK_TEMPLATE_T.DEFINITION_NAME, TASK_TEMPLATE_T.TARGET_NAMESPACE, TASK_TEMPLATE_T.IS_AD_HOC, TASK_TEMPLATE_T.IS_INLINE, TASK_TEMPLATE_T.KIND, TASK_TEMPLATE_T.NAME, TASK_TEMPLATE_T.NAMESPACE, TASK_TEMPLATE_T.PRIORITY, TASK_TEMPLATE_T.STATE, TASK_TEMPLATE_T.SUPPORTS_AUTO_CLAIM, TASK_TEMPLATE_T.SUPPORTS_CLAIM_SUSPENDED, TASK_TEMPLATE_T.SUPPORTS_DELEGATION, TASK_TEMPLATE_T.SUPPORTS_SUB_TASK, TASK_TEMPLATE_T.SUPPORTS_FOLLOW_ON_TASK, TASK_TEMPLATE_T.TYPE, TASK_TEMPLATE_T.VALID_FROM, TASK_TEMPLATE_T.AUTONOMY
 FROM TASK_TEMPLATE_T;

-- View: Escalation
CREATE VIEW ESCALATION(ESIID, ACTION, ACTIVATION_STATE, ACTIVATION_TIME, ESCALATION_TIME, AT_LEAST_EXP_STATE, ESTID, FIRST_ESIID, INCREASE_PRIORITY, NAME, STATE, TKIID ) AS
 SELECT ESCALATION_INSTANCE_T.ESIID, ESCALATION_INSTANCE_T.ACTION, ESCALATION_INSTANCE_T.ACTIVATION_STATE, ESCALATION_INSTANCE_T.ACTIVATION_TIME, ESCALATION_INSTANCE_T.ESCALATION_TIME, ESCALATION_INSTANCE_T.AT_LEAST_EXPECTED_STATE, ESCALATION_INSTANCE_T.ESTID, ESCALATION_INSTANCE_T.FIRST_ESIID, ESCALATION_INSTANCE_T.INCREASE_PRIORITY, ESCALATION_INSTANCE_T.NAME, ESCALATION_INSTANCE_T.STATE, ESCALATION_INSTANCE_T.TKIID
 FROM ESCALATION_INSTANCE_T;

-- View: EscTempl
CREATE VIEW ESC_TEMPL(ESTID, FIRST_ESTID, PREVIOUS_ESTID, TKTID, CONTAINMENT_CTX_ID, NAME, ACTIVATION_STATE, AT_LEAST_EXP_STATE, INCREASE_PRIORITY, ACTION ) AS
 SELECT ESCALATION_TEMPLATE_T.ESTID, ESCALATION_TEMPLATE_T.FIRST_ESTID, ESCALATION_TEMPLATE_T.PREVIOUS_ESTID, ESCALATION_TEMPLATE_T.TKTID, ESCALATION_TEMPLATE_T.CONTAINMENT_CONTEXT_ID, ESCALATION_TEMPLATE_T.NAME, ESCALATION_TEMPLATE_T.ACTIVATION_STATE, ESCALATION_TEMPLATE_T.AT_LEAST_EXPECTED_STATE, ESCALATION_TEMPLATE_T.INCREASE_PRIORITY, ESCALATION_TEMPLATE_T.ACTION
 FROM ESCALATION_TEMPLATE_T;

-- View: EscTemplDesc
CREATE VIEW ESC_TEMPL_DESC(ESTID, LOCALE, TKTID, DISPLAY_NAME, DESCRIPTION ) AS
 SELECT ESC_TEMPL_LDESC_T.ESTID, ESC_TEMPL_LDESC_T.LOCALE, ESC_TEMPL_LDESC_T.TKTID, ESC_TEMPL_LDESC_T.DISPLAY_NAME, ESC_TEMPL_LDESC_T.DESCRIPTION
 FROM ESC_TEMPL_LDESC_T;

-- View: TaskTemplCProp
CREATE VIEW TASK_TEMPL_CPROP(TKTID, NAME, DATA_TYPE, STRING_VALUE ) AS
 SELECT TASK_TEMPL_CPROP_T.TKTID, TASK_TEMPL_CPROP_T.NAME, TASK_TEMPL_CPROP_T.DATA_TYPE, TASK_TEMPL_CPROP_T.STRING_VALUE
 FROM TASK_TEMPL_CPROP_T;

-- View: EscTemplCProp
CREATE VIEW ESC_TEMPL_CPROP(ESTID, NAME, TKTID, DATA_TYPE, VALUE ) AS
 SELECT ESC_TEMPL_CPROP_T.ESTID, ESC_TEMPL_CPROP_T.NAME, ESC_TEMPL_CPROP_T.TKTID, ESC_TEMPL_CPROP_T.DATA_TYPE, ESC_TEMPL_CPROP_T.STRING_VALUE
 FROM ESC_TEMPL_CPROP_T;

-- View: TaskTemplDesc
CREATE VIEW TASK_TEMPL_DESC(TKTID, LOCALE, DESCRIPTION, DISPLAY_NAME ) AS
 SELECT TASK_TEMPL_LDESC_T.TKTID, TASK_TEMPL_LDESC_T.LOCALE, TASK_TEMPL_LDESC_T.DESCRIPTION, TASK_TEMPL_LDESC_T.DISPLAY_NAME
 FROM TASK_TEMPL_LDESC_T;

-- View: TaskCProp
CREATE VIEW TASK_CPROP(TKIID, NAME, DATA_TYPE, STRING_VALUE ) AS
 SELECT TASK_INST_CPROP_T.TKIID, TASK_INST_CPROP_T.NAME, TASK_INST_CPROP_T.DATA_TYPE, TASK_INST_CPROP_T.STRING_VALUE
 FROM TASK_INST_CPROP_T;

-- View: TaskDesc
CREATE VIEW TASK_DESC(TKIID, LOCALE, DESCRIPTION, DISPLAY_NAME ) AS
 SELECT TASK_INST_LDESC_T.TKIID, TASK_INST_LDESC_T.LOCALE, TASK_INST_LDESC_T.DESCRIPTION, TASK_INST_LDESC_T.DISPLAY_NAME
 FROM TASK_INST_LDESC_T;

-- View: EscalationCProp
CREATE VIEW ESCALATION_CPROP(ESIID, NAME, DATA_TYPE, STRING_VALUE ) AS
 SELECT ESC_INST_CPROP_T.ESIID, ESC_INST_CPROP_T.NAME, ESC_INST_CPROP_T.DATA_TYPE, ESC_INST_CPROP_T.STRING_VALUE
 FROM ESC_INST_CPROP_T;

-- View: TaskHistory
CREATE VIEW TASK_HISTORY(TKIID, ESIID, PARENT_TKIID, EVENT, REASON, EVENT_TIME, NEXT_TIME, PRINCIPAL, WORK_ITEM_KIND, FROM_ID, TO_ID ) AS
 SELECT TASK_HISTORY_T.TKIID, TASK_HISTORY_T.ESIID, TASK_HISTORY_T.PARENT_TKIID, TASK_HISTORY_T.EVENT, TASK_HISTORY_T.REASON, TASK_HISTORY_T.EVENT_TIME, TASK_HISTORY_T.NEXT_TIME, TASK_HISTORY_T.PRINCIPAL, TASK_HISTORY_T.WORK_ITEM_KIND, TASK_HISTORY_T.FROM_ID, TASK_HISTORY_T.TO_ID
 FROM TASK_HISTORY_T;

-- View: EscalationDesc
CREATE VIEW ESCALATION_DESC(ESIID, LOCALE, DESCRIPTION, DISPLAY_NAME ) AS
 SELECT ESC_INST_LDESC_T.ESIID, ESC_INST_LDESC_T.LOCALE, ESC_INST_LDESC_T.DESCRIPTION, ESC_INST_LDESC_T.DISPLAY_NAME
 FROM ESC_INST_LDESC_T;

-- View: TaskAuditLog
CREATE VIEW TASK_AUDIT_LOG(ALID, AUDIT_EVENT, CONTAINMENT_CTX_ID, ESIID, ESTID, EVENT_TIME, FAULT_NAME, FAULT_TYPE_NAME, FAULT_NAME_SPACE, FOLLOW_ON_TKIID, NAME, NAMESPACE, NEW_USER, OLD_USER, PARENT_CONTEXT_ID, PARENT_TASK_NAME, PARENT_TASK_NAMESP, PARENT_TKIID, PRINCIPAL, TASK_KIND, TASK_STATE, TKIID, TKTID, TOP_TKIID, VAILD_FROM, WORK_ITEM_REASON, USERS, DESCRIPTION, MESSAGE_DATA ) AS
 SELECT TASK_AUDIT_LOG_T.ALID, TASK_AUDIT_LOG_T.AUDIT_EVENT, TASK_AUDIT_LOG_T.CONTAINMENT_CONTEXT_ID, TASK_AUDIT_LOG_T.ESIID, TASK_AUDIT_LOG_T.ESTID, TASK_AUDIT_LOG_T.EVENT_TIME_UTC, TASK_AUDIT_LOG_T.FAULT_NAME, TASK_AUDIT_LOG_T.FAULT_TYPE_NAME, TASK_AUDIT_LOG_T.FAULT_NAMESPACE, TASK_AUDIT_LOG_T.FOLLOW_ON_TKIID, TASK_AUDIT_LOG_T.NAME, TASK_AUDIT_LOG_T.NAMESPACE, TASK_AUDIT_LOG_T.NEW_USER, TASK_AUDIT_LOG_T.OLD_USER, TASK_AUDIT_LOG_T.PARENT_CONTEXT_ID, TASK_AUDIT_LOG_T.PARENT_TASK_NAME, TASK_AUDIT_LOG_T.PARENT_TASK_NAMESPACE, TASK_AUDIT_LOG_T.PARENT_TKIID, TASK_AUDIT_LOG_T.PRINCIPAL, TASK_AUDIT_LOG_T.TASK_KIND, TASK_AUDIT_LOG_T.TASK_STATE, TASK_AUDIT_LOG_T.TKIID, TASK_AUDIT_LOG_T.TKTID, TASK_AUDIT_LOG_T.TOP_TKIID, TASK_AUDIT_LOG_T.VALID_FROM_UTC, TASK_AUDIT_LOG_T.WI_REASON, TASK_AUDIT_LOG_T.USERS, TASK_AUDIT_LOG_T.DESCRIPTION, TASK_AUDIT_LOG_T.MESSAGE_DATA
 FROM TASK_AUDIT_LOG_T;

-- update schema version to: 620
-- (a warning for the following statement can be ignored)
DELETE FROM SCHEMA_VERSION WHERE SCHEMA_VERSION = 620;
INSERT INTO SCHEMA_VERSION (SCHEMA_VERSION, DATA_MIGRATION) VALUES( 620, 1 );

