"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: installRAR.py
	
	description: Install a Resource Adapter Archive to a WAS node
"""
import sys
from ConfigWriter import ConfigWriterException

def installRar (nodeName, rarFile, rarName ):

	# read available installable apps
	cellName = AdminControl.getCell()
	node = AdminConfig.getid( '/Cell:' + cellName + '/Node:' + nodeName + '/' )
	rarOption = "-rar.name " + rarName

	rarExists = checkIfRarExists(cellName,nodeName,rarName)
	if (rarExists == "true"):
		## delete the resource adapter
		print "Resource Adapter with name "+rarName+" already exists at /Node:"+nodeName+"/"
		print ""
		print "Removing "+rarName+" ..."
		radapters = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/J2CResourceAdapter:"+rarName+"/" ).split(lineSeparator)
		for radapter in radapters:
			AdminConfig.remove(radapter)
		#endFor

		print "---  Installing Resource Adapter archive: " + rarName
		newId = AdminConfig.installResourceAdapter(rarFile, nodeName, rarOption)
	else:
		print "---  Installing Resource Adapter archive: " + rarName
		newId = AdminConfig.installResourceAdapter(rarFile, nodeName, rarOption)
	#endElse

	# check if we were successful
	rarExists = checkIfRarExists(cellName,nodeName,rarName)
	if (rarExists):
		print "Resource adapter archive "+rarName+" installed successfully."
	else:
		msg = "Failed to install resource adapter archive ="+rarName
		raise ConfigWriterException(msg)
	#endElse 
#endDef 

def checkIfRarExists (cellName,nodeName,rarName):
	print ""
	print "Checking if Resource Adapter "+rarName+" exists at /Node:"+nodeName+"/"
	global AdminConfig
	rarExists = "true"
	radapter = AdminConfig.getid("/Cell:"+cellName+"/Node:"+nodeName+"/J2CResourceAdapter:"+rarName+"/" ).strip()
	if (len(radapter) == 0):
		print "Did not find Resource Adapter named "+rarName+" at /Node:"+nodeName+"/"
		rarExists = "false"
	#endIf 
	return rarExists
#endDef 


###### MAIN ######

# parse the options into optDict
optDict, args = SystemUtils.getopt( sys.argv, 'rarProps:;rarFile:;scope:;scopename:;mode:;script_name:' )

# parse the properties into props
props = SystemUtils.getPropertyDict( optDict['rarProps'] )
scopename = optDict['scopename']

if (props.has_key("RAR_NAME")):
	rarName = props["RAR_NAME"]
else:
	raise("RAR_NAME must be defined in " + optDict['rarProps'])
#endIf

installRar( optDict['scopename'], optDict['rarFile'], rarName )

# save config 
print "Saving Configuration and Synchronizing Nodes"
AdminHelper.saveAndSyncCell( )
