"""
        Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

        
        File name: AddRepository.py
        
        Contains methods to add Installation Manager repositories to the
        response file.
"""
#from popen2 import popen3
import os.system
import tempfile
import os.path
import re
import glob

class PAKHelper:
	"""
	Helper class to find the installed FEPs and the patches for them.  This 
	is needed because the upgrade mechanism for WAS/FEP requires that WAS
	and the FEPs be at the same level, and also that the upgrade be
	carried out in the same 'UpdateManager' session.  So we have to find
	out the FEPs installed and install patches for them along with the WAS
	patch at the same level.
	"""
	installedFEPList = []
	FPPathList = []
	PAKFileList = []
	transTable = { 'EJB3' : 'ejb3', 'WEBSERVICES' : 'websv', 'ND': None , 'ESB' : None, 'WPS': None, 'WBI': None}
	tempFileName = ""
	fepToPatchFound = 0

	def __init__(self, tempFileVal, mediaHome, wasVersion, FEP,
			patchNum ):
		"""
		Constructor/entry point for this class.
		"""
		self.tempFileName = tempFileVal
		"""
		Commenting runScript method since the exec command is been taken care of in ant xml file.
		"""
		#self.runScript(scriptPath)
		self.generateInstalledFEPList()
		self.generateFPPaths(mediaHome, wasVersion, FEP, patchNum)
		self.generatePAKFileList()
		os.unlink(self.tempFileName)
	#enDef
	
	"""
	This method have been deprecated since it uses os.system which doesn't support
	all platforms like Windows 2003, Windows Vista etc.
	"""
	def runScript(self, scriptPath):
		"""
		Runs the "versionInfo.{sh,bat}" script to find the installed 
		FEPs.  The output is channelled back using a pipe (popen3()), 
		and parsed using other functions in this class.
		"""
		# self.scriptIn, self.scriptOut, self.scriptErr = \
		#		popen3(scriptPath, "r")
		os.system(scriptPath + ' > ' +  self.tempFileName)
	#enDef


	def generateInstalledFEPList(self):
		"""
		Parses the output of versionInfo.sh and generates the list of
		installed FEPs.  The list is stored to the class variable
		installedFEPList.
		"""
		tempFileHandle = open(self.tempFileName, 'r')
		for line in tempFileHandle.readlines():
			match = re.search('\w+[\t ]+installed', line)
			if match != None:
				ifep = re.split('[ \t]+', line)[0]
				self.installedFEPList.append(ifep)
			#endIf
		#endFor
		print "Installed FEPs are:", self.installedFEPList
		tempFileHandle.close()

		# Close the pipes created by popen3() here, now that we are
		# done with them.
		# self.scriptIn.close()

		# Some dicey error checking.
		# r = self.scriptOut.close()
		# if r != None:
		# 	print "Script returned: ", r
		# 	for line in scriptErr.readlines():
		# 		print line.rstrip()
		# self.scriptErr.close()
	#enDef
	
	def generateFPPaths(self, mediaHome, wasVersion, FEP, fpNum):
		"""
		Generate the Fix-Pack directory names we require.  Store them 
		in the class variable FPPathList.
		"""
		basePath = os.path.abspath(mediaHome)
		wasFPPath = os.path.join(basePath, "patches", 
				"was" + wasVersion + "_" + fpNum)
		self.FPPathList.append(wasFPPath)
		for fep in self.installedFEPList:
			translatedFEP = self.transTable[fep]
			if translatedFEP == None:
				continue
			#endIf
			if translatedFEP == FEP:
				self.fepToPatchFound = 1
			#endIf
			FEPPatchPath = os.path.join(basePath, "feature_packs",
					translatedFEP, "patches",
					translatedFEP + "_" + fpNum)
			self.FPPathList.append(FEPPatchPath)
		#endFor
	#enDef
	
	def generatePAKFileList(self):
		"""
		Generate the list of PAK files in the directories we have.
		"""
		if self.fepToPatchFound != 1:
			return
		#endIf
		for directory in self.FPPathList:
			if not os.path.isdir(directory):
				# Silently ignore this entry and try to
				# proceed with others.
				continue
			#endIf
			pakfiles = glob.glob(os.path.join(directory, 
				"*.[Pp][Aa][Kk]"))
			for pakfile in pakfiles:
				# Silenty ignore unless regular file or symlink
				# to one.
				if os.path.isfile(pakfile):
					self.PAKFileList.append(pakfile)
			#endFor
		#endFor
	#enDef
	
	def getPAKFileLine(self):
		"""
		Return the line we require in ANT.  This is generated from the
		list of PAK files and parsed into a string with appropriate
		leader and trailer.
		"""
		PAKFileLine = "-W maintenance.package=\""
		allNones = 1
		if len(self.PAKFileList) > 0:
			for pakfile in self.PAKFileList:
				if os.path.isfile(pakfile):
					PAKFileLine = PAKFileLine + \
							pakfile + ";"
					allNones = 0
			PAKFileLine = PAKFileLine + "\""
		#endIf
		if allNones == 1 or self.fepToPatchFound != 1:
			return "Not found"
		#endIf
		return PAKFileLine
	#enDef

# Usage should be something like below:
# print PAKHelper("/opt/IBM/WebSphere/bin/versionInfo.sh", "/opt/buildforge/rafw/media", "70", "fp16").getPAKFileLine()
