"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: BFWCommand.py
	
	Encapsulates the logic required to run a typical WAS configuration command
"""

import sys
try:
	sys.modules['AdminConfig'] = AdminConfig
except:
	pass
import AdminConfig
import java

from Logger import _Logger
from SystemUtils import SystemUtils
from ConfigReader import ConfigReader
from ConfigFileWriter import ConfigFileWriter
from ConfigWriter import ConfigWriterException 
from ConfigWriter import ConfigWriter
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

from SystemUtils import *
from AdminHelper import *
from ConfigMediator import *
import Globals

############## CLASS :: _BFWCommand  #####################
# Encapsulates the logic required to run a typical       #
# WAS configuration command                              #
#                                                        #
##########################################################
class _BFWCommand:

	def __init__(self):
		self.LOGGER = _Logger("BFWCommand", MessageManager.RB_WEBSPHERE_WAS)
	#endDef

	"""
		Description:                                        
			based on the supplied mode, either do import       
			export or compare on the given WAS configuration 
			type.                                           
	                                                       
		Inputs:                                             
	                                                       
	"""
	def execute(self, bfwArgs, typeNames):
		
		# parse the options into optDict
		optDict, args = SystemUtils.getopt( bfwArgs, 'scope:;properties:;nodename:;scopename:;mode:;marker:' )
		
		marker = optDict['marker']
		propFile = optDict['properties'] 
		
		# get scope
		scope = AdminHelper.buildScope( optDict )
		
		scopeType=optDict['scope']
		excludeTypes = ['builtin']
		
		mode = optDict['mode']
		if (mode == MODE_EXECUTE):
		
			print "Creating types in scope: " + scope + " types: " + str(typeNames)
			ConfigMediator.createConfig(scope, scopeType, propFile, marker, typeNames)
			AdminHelper.saveAndSyncCell()
		
		elif (mode == MODE_IMPORT):
			print "Importing types in scope: " + scope + " types: " + str(typeNames)
			ConfigMediator.importConfig(scope, scopeType, propFile, marker, typeNames)
		
		elif (mode == MODE_COMPARE):
			print "Comparing types in RAFW and WAS for scope: " + scope + " types: " + str(typeNames)
			ConfigMediator.compareConfig(scope, scopeType, propFile, marker, typeNames, excludeTypes)
			
		elif (mode == MODE_AUGMENT):
			print "Augmenting types in scope: " + scope + " types: " + str(typeNames)
			scopeId = AdminConfig.getid(scope)
			ConfigMediator.augmentConfigUsingParentId(scopeId, scopeType, propFile, marker, typeNames, excludeTypes)
			AdminHelper.saveAndSyncCell()
		else:
			print "Unsupported MODE supplied: " + mode
		#endIf
	#endDef
	
#endClass

def export(optDict=None):
	marker = optDict['marker']
	propFile = optDict['properties'] 
	typeNames=[optDict['type']]
	# get scope
	scope = optDict['wasscopetype']
	
	scopeType=optDict['scope']
	#Leave this in  here because this was there from before even though the data_type_mappings
	#doesn't have it to make sure we don't skip builtin from quick capture
	excludeTypes = ['builtin']
	
	ConfigMediator.createConfig(scope, scopeType, propFile, marker, typeNames)
#enddef

## define global singletons
BFWCommand = _BFWCommand()
