"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: ConfigFileWriter.py
	
	This contains methods and classes to facilitate writing WebSphere configuration data.
"""

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from JavaConversionHelper import _JavaConversionHelper
from org.apache.commons.lang import StringEscapeUtils

import com.ibm.rational.rafw.wsadmin.websphere.ConfigFileWriter as JavaConfigFileWriter

############## CLASS :: ConfigFileWriter #################
# Use this class to write configuration files inside     #
# RAFW                                                   #
#                                                        #
##########################################################
class ConfigFileWriter:

	######################################################
	## This constructor optionally defines the velocity ##
	## template that should                             ##
	## be used for processing                           ##
	######################################################
	def __init__(self, template="genericXML.vm"):
		self.template = template
		self.LOGGER = _Logger("ConfigFileWriter", MessageManager.RB_WEBSPHERE_WAS)
		self.javaConversionHelper = _JavaConversionHelper(self._filterXMLString)
		self.jConfigFileWriter = JavaConfigFileWriter(template)
	#endDef
	
	######################################################
	## make the inputString safe for XML                ##
	## we should consider moving to                     ## 
	##   org.apache.commons.lang.StringEscapeUtils      ##
	## which we already have license for in ies36       ##
	######################################################
	def _filterXMLString(self, inputString):
		# ampersand must always be first
		outputString = inputString.replace("&", "&amp;")
		outputString = outputString.replace("<", "&lt;")
		outputString = outputString.replace(">", "&gt;")
		outputString = outputString.replace("\"", "&quot;")
		outputString = outputString.replace("\t", "&#9;")
		return outputString
	#endDef

	######################################################
	## Replaces the entire contents of file with supplied 
	## data using velocity template
	######################################################
	def replaceFile (self, xmlFile, data):
		
		self.LOGGER.traceEnter([xmlFile])
		javaData =  self.javaConversionHelper.mapToJava(data);
		self.jConfigFileWriter.replaceFile(xmlFile, javaData)
		self.LOGGER.traceExit()

	#endDef

	######################################################
	## Injects supplied data into a file
	##
	## xmlFile - the file to inject data into
	## data - the data
	## marker - string that denotes the container node name
	##               whose contents will be replaced
	## 
	######################################################
	def processBasicFile (self, xmlFile, data, marker):
		self.LOGGER.traceEnter([xmlFile, marker])
		javaData =  self.javaConversionHelper.mapToJava(data);
		self.jConfigFileWriter.processBasicFile(xmlFile, marker, javaData)
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Migrates the XML file from the old format to the new format
	
		xmlFile - the file to migrate
	 	marker  - string that denotes the container node name
	 	<!-- BEGIN marker --> will be replaced with <RAFW_marker> 
	 	and <!-- END marker --> will be replaced with </RAFW_marker>
	 	
	 	@return: true (1) if the file contains a valid RAFW_marker element after migration
	"""
	def migrateFile (self, xmlFile, marker):
		
		self.LOGGER.traceEnter([xmlFile, marker])
		retVal = self.jConfigFileWriter.migrateFile(xmlFile, marker)
		self.LOGGER.traceExit(retVal)
		return retVal
	#endDef
#endClass

## define global singletons
GenericConfigFileWriter = ConfigFileWriter()

