"""
	Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 

	
	File name: WasConfig.py
	
	Encapsulates useful utilities used for WebSphere specific configuration
"""

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from AdminHelper import *
from ConfigReader import ConfigReader

import AdminConfig
import AdminTask

"""
CLASS :: _WasConfig
Encapsulates useful utilities used for WebSphere      
specific configuration                                
"""
class _WasConfig:
	
	def __init__(self):
		self.LOGGER = _Logger("WasConfig", MessageManager.RB_WEBSPHERE_WAS)
	#endDef

	############ updateContainerService() ######################
	# update a container service of a server                                                  
	###################################################################
	def updateContainerService(self, properties, server, serviceType, serviceSettings, propertySettings, threadPoolSettings=""):
		service = AdminConfig.list(serviceType, server)
	
		attrs=AdminHelper.buildAttrs(properties, serviceSettings)
		AdminConfig.modify(service, attrs)
	
		props=AdminConfig.list("Property", service).split(newline)
	
		propList={}
		for prop in props:
			if (prop == None or prop == ''):
				break
			#endif
	
			name=AdminConfig.showAttribute(prop, "name")
			propList[name]=prop
		#endfor
			
		try:
			children = properties['children']
			for elmt in children:
				if (elmt.has_key('threadPool')):
					if (threadPoolSettings != ""):
						threadPoolElmt=elmt['threadPool']
						attrs=AdminHelper.buildAttrs(threadPoolElmt, threadPoolSettings)
						threadPool=AdminConfig.showAttribute(service, "threadPool")
						AdminConfig.modify(threadPool, attrs)
					#endif
									
				elif (elmt.has_key('property')):
					propElmt=elmt['property']
					attrs=AdminHelper.buildAttrs(propElmt, propertySettings)
	
					name=propElmt['name']	
					try:
						prop=propList[name]
					except:
						prop=""
					#endtry
						
					if (prop != ""):
						AdminConfig.remove(prop)
					#endif
				
					# Create the property	
					AdminConfig.create('Property', service, attrs)				
	
				#endif
			#endfor				
		except KeyError:
			pass
		#endtry
	
	#endDef
	

	# note that any changes to this action will effect wp_common_configure_update_wpconfig_ports since this action assumes a certain order
	def updateServerPorts(self, wasVersion, nodeName, serverName, serverNumber=1, transportInc=0, transportStartPort=0):
		self.LOGGER.traceEnter([wasVersion, nodeName, serverName, serverNumber, transportInc, transportStartPort])
		endPointNames = [ "BOOTSTRAP_ADDRESS", 
		"CSIV2_SSL_SERVERAUTH_LISTENER_ADDRESS",
		"CSIV2_SSL_MUTUALAUTH_LISTENER_ADDRESS", 
		"DCS_UNICAST_ADDRESS",
		"ORB_LISTENER_ADDRESS", 
		"SAS_SSL_SERVERAUTH_LISTENER_ADDRESS",
		"SIB_ENDPOINT_ADDRESS", 
		"SIB_ENDPOINT_SECURE_ADDRESS",
		"SIB_MQ_ENDPOINT_ADDRESS", 
		"SIB_MQ_ENDPOINT_SECURE_ADDRESS",
		"SOAP_CONNECTOR_ADDRESS",
		"WC_adminhost",
		"WC_defaulthost", 
		"WC_adminhost_secure", 
		"WC_defaulthost_secure" ]
		
		wasVersInt = int(wasVersion)
		#  The following are added in WAS 6.1
		if (wasVersInt > 60):
			endPointNames.append("SIP_DEFAULTHOST")
			endPointNames.append("SIP_DEFAULTHOST_SECURE")
		#endIf
		
		#  The following are added in WAS 7.0
		if (wasVersInt > 61):
			endPointNames.append("IPC_CONNECTOR_ADDRESS")
		#endIf
		
		# While working on defect 27130, we learned that the ports
		# OVERLAY_UDP_LISTENER_ADDRESS and OVERLAY_TCP_LISTENER_ADDRESS
		# got added in WAS 8.5 and not in WAS 8.0. We also verified that
		# "create cluster" library step fails not only for Portal 8.0
		# but also for WAS 8.0 with the same error message - 
		# 'The endpoint OVERLAY_UDP_LISTENER_ADDRESS is not defined'
		if (wasVersInt > 80):
			endPointNames.append("OVERLAY_UDP_LISTENER_ADDRESS")
			endPointNames.append("OVERLAY_TCP_LISTENER_ADDRESS")
		#endIf
		
		# Now we're going to adjust the tcp transports if TRANSPORT_STARTING_POINT was defined
		## we cannot update port numbers for the second cluster member if transportInc is 0
		if (transportStartPort > 0 and (serverNumber == 1 or transportInc > 0) ):
			myPort = ((serverNumber-1) * transportInc) + transportStartPort
			for sEPoint in endPointNames:
				sAttrs = '[-nodeName ' + nodeName + ' -endPointName ' + sEPoint + ' -port ' + str(myPort) + ' -modifyShared]'
				print "--## Setting %10s:%40s == %5s ##--" % (serverName, sEPoint, str(myPort))
				AdminTask.modifyServerPort(serverName, sAttrs)
				myPort = myPort+1
			#endFor
		else:
			if (transportStartPort == 0):
				print "TRANSPORT_STARTING_POINT is 0, ports are left at the defaults"
			if (serverNumber > 1 and transportInc == 0):
				print "TRANSPORT_NODE_INCREMENTOR is 0, ports are left at the defaults"
		#endif
		self.LOGGER.traceExit()
	#endDef
	
	"""
	Returns 1 if WIMUserRegistry is the activeUserRegistry configured for this cell
	else returns 0
	"""
	def isWimEnabled(self, optDict):
		self.LOGGER.traceEnter([optDict])
		scope = AdminHelper.buildScope( optDict )
		scopeId = AdminConfig.getid(scope)
		for objId in AdminConfig.list("Security", scopeId).split(newline):
			if (len(objId) > 0):
				activeUR = AdminConfig.show(objId, "activeUserRegistry")
				if (len(activeUR) > 0):
					configReader = ConfigReader()
					wasType = configReader.getWASType(activeUR)
					if (wasType == "WIMUserRegistry"):
						self.LOGGER.traceExit(1)
						return 1
					#endIf
				#endIf
			#endIf
		#endFor
		# if there was no object id for security, then there is no security config
		# wim is not enabled
		self.LOGGER.traceExit(0)
		return 0
	#endDef
#endClass

## define global singletons
WasConfig = _WasConfig()	

