'''
Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Alden Taylor
'''
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager
from com.ibm.rational.rafw.wsadmin.util import WsadminStringUtils
from java.util import ArrayList
from java.io import File
from existingcell.Node import Node

from existingcell.Constants import NEWLINE
from existingcell.AbstractCellProcessor import AbstractCellProcessor
from existingcell.Server import Server

import AdminConfig
import AdminTask
import AdminControl

"""
	Processor that gathers WAS and WP topology information from a runtime configuration
"""
class WasCellProcessor(AbstractCellProcessor):
	
	def __init__(self, cellDefinitionReader):
		AbstractCellProcessor.__init__(self, cellDefinitionReader)
		self.LOGGER = _Logger("WasCellProcessor", MessageManager.RB_WEBSPHERE_WAS)
		self.environmentDict = {}
		self.clusterNum = 0
		self.clusters=""
		self.cellType="WAS"
		self.isStandalone = 0
		## read node data
		self.managedNodeNum = 0
		self.nodes = []
		self.discoveredNodes = {}
		self.discoveredServers = {}
		self.platform = ""
	#endDef
	
	"""
		Retrieves the needed properties related to a node which contains an 
		application server.
		
		Properties are stored in self.environmentDict
	"""
	def _processManagedNode(self, node):
		self.LOGGER.traceEnter()
		nodeName = node.getNodeName()
		nodeAttrs = self.cellDefinitionReader.adminAgentManagedNodesDict[nodeName]
		for nodeAttr in nodeAttrs:
			if (nodeAttr[0] == "Port"):
				nodePort = nodeAttr[1]
			elif (nodeAttr[0] == "Host"):
				nodeHost = nodeAttr[1]
			elif (nodeAttr[0] == "ProfileWASHome"):
				nodeWASHome = nodeAttr[1]
			elif (nodeAttr[0] == "ProfileCell"):
				nodeCellName = nodeAttr[1]
			elif (nodeAttr[0] == "ProfileRoot"):
				nodeProfileRoot = nodeAttr[1]
				profileHomeSplit = nodeProfileRoot.split('/')
				nodeProfileName = profileHomeSplit[len(profileHomeSplit) -1]
			#endIf
		#endFor
		self._setNodeProperty(node, "_CELL_NAME", nodeCellName)
		self._setNodeProperty(node, "_HOST_NAME", nodeHost)
		self._setNodeProperty(node, "_NODE_NAME", nodeName)
		self._setNodeProperty(node, "_PROFILE_NAME", nodeProfileName)
		self._setNodeProperty(node, "_WAS_HOME", nodeWASHome)
		self._setNodeProperty(node, "_WAS_PROF_HOME", nodeProfileRoot)
		self._setNodeProperty(node, "_NODE_TYPE", node.getDataType())
		self._setNodeProperty(node, "_SOAP_PORT", nodePort)
		self._setNodeProperty(node, "_STARTING_POINT", "")
		if (self.platform == "zos"):
			self._setNodeProperty(node, "_OS", "ZOS")
		else:
			self._setNodeProperty(node, "_OS", "Non-ZOS")
		#endIf
		self.LOGGER.traceExit()
	#endDef
	
	"""
		Retrieves the needed properties related to a node which contains an 
		application server.
		
		Properties are stored in self.environmentDict
	"""
	def _processAppServerNode(self, node):
		nodeName = node.getNodeName()
		self.LOGGER.traceEnter()
		nodeVars = node.getNodeVariables()
		if (self.isStandalone):
			self.environmentDict["BASE_STARTING_POINT"] = ""
		else:
			self._setNodeProperty(node, "_BASE_STARTING_POINT", "")
		self._setNodeProperty(node, "_HOST_NAME", AdminConfig.showAttribute(node.getConfigId(), "hostName"))
		self._setNodeProperty(node, "_NODE_NAME", nodeName)
		self._setNodeProperty(node, "_PROFILE_NAME", node.getProfileName())
		
		if (nodeVars.has_key("WAS_INSTALL_ROOT")):
			self._setNodeProperty(node, "_WAS_HOME", nodeVars['WAS_INSTALL_ROOT'])
		else:
			raise "VariableSubstitutionEntry with name='WAS_INSTALL_ROOT' is not defined in the WebSphere configuration."
		#endIf
		if (nodeVars.has_key("USER_INSTALL_ROOT")):
			self._setNodeProperty(node, "_WAS_PROF_HOME", nodeVars['USER_INSTALL_ROOT'])
		else:
			self.environmentDict["ND_WAS_PROF_HOME"] = nodeVars['WAS_INSTALL_ROOT'] + "/profiles/" + node.getProfileName()
		#endIf
		
		nodeType=node.getDataType()
		
		if (nodeVars.has_key("WPS_HOME")):
			# this node is a portal node
			if(self.isStandalone):
				nodeType = Node.WP_BASE_NODE
			else:
				nodeType = Node.WP_MANAGED_NODE
			## If we encounter a WP node, than the cell type is WP
			self.cellType="WP"
			wpHome = nodeVars["WPS_HOME"]
			if (self.platform == "zos"):
				# look for PROFILE_HOME/PortalServer
				wpHome = nodeVars['USER_INSTALL_ROOT'] + "/PortalServer"
				zosPortalHome = File(wpHome)
				if (zosPortalHome.exists()):
					self.LOGGER.log("CRWWA0135I", wpHome)
				else:
					self.LOGGER.log("CRWWA0136W", wpHome)
				#endIf
			#endIf
			self._setNodeProperty(node, "_WP_HOME", wpHome)
			self._setNodeProperty(node, "_WP_SETUP_TYPE", "full")
			self._setNodeProperty(node, "_WP_STARTING_POINT", "")
			# if we haven't identified a primary node
			if (not self.environmentDict.has_key("PRIMARY_NODE")):
				self.environmentDict["PRIMARY_NODE"] = nodeName
			#endIf
			self.environmentDict["WP_CONFIGURE_WCM"] = "true"
		#endIf
		
		# write it into the environmentDict
		self._setNodeProperty(node, "_NODE_TYPE", nodeType)
		
		if (self.isStandalone):
			portsByServer = node.getPorts()
			if (portsByServer.has_key("server1")):
				## user server1's SOAP port
				ports = portsByServer["server1"]
			else:
				## pick the first ports definition
				ports = portsByServer.values()[0]
			#endIf
			if (ports.has_key("SOAP_CONNECTOR_ADDRESS")):
				self._setNodeProperty(node, "_SOAP_PORT", ports["SOAP_CONNECTOR_ADDRESS"])
			else:
				self._setNodeProperty(node, "_SOAP_PORT", "8880")
			#endIf
		#endIf
		if (self.platform == "zos"):
			self._setNodeProperty(node, "_OS", "ZOS")
		else:
			self._setNodeProperty(node, "_OS", "Non-ZOS")
		#endIf
		self.LOGGER.traceExit()
	#endDef

	"""
		Sets a property in the environment dictionary, also used by sub classes
	"""	
	def _setNodeProperty(self, node, keySuffix, value):
		key = node.getPrefix() + keySuffix
		self.environmentDict[key] = value
	#endDef
	
	"""
		Called by CellDefinitionReader when a cell is discovered. 
	"""	
	def cellDiscovered(self, cellId):
		cellType=AdminConfig.showAttribute(cellId,'cellType')
		self.LOGGER.log("CRWWA0137I", cellType)
		if (cellType == "STANDALONE"):
			self.isStandalone = 1
		elif (cellType == "DISTRIBUTED"):
			self.isStandalone = 0
		else:
			self.LOGGER.log("CRWWA0138I")
			self.isStandalone = 0
		#endIf
	#endDef
	
	"""
		Called by CellDefinitionReader when a node is discovered in the cell configuration
	"""	
	def nodeDiscovered(self, nodeId):
		nodeConfig = Node(nodeId, self.isAdminAgentCell)
		if (self.isAdminAgentCell == "true"):
			self.managedNodeNum += 1;
			nodeConfig.managedNodeNum = self.managedNodeNum
			self.nodes.append(nodeConfig.getNodeName())
		#endIf
		self.discoveredNodes[nodeId] = nodeConfig
		self.LOGGER.log("CRWWA0139I", nodeConfig.getNodeName())
	#endDef
	
	"""
		Called by CellDefinitionReader when a server is discovered in the node configuration
	"""	
	def serverDiscovered(self, nodeId, serverId, serverName=""):
		node = self.discoveredNodes[nodeId]
		server = Server(serverId, self.platform, self.isAdminAgentCell)
		
		if(serverName != ""):
			server.serverName = serverName
		
		# figure out what kind of server we got here
		server.identifyServer(node, self.managedNodeNum, serverName)
		
		# figure out the type of node based on the server
		nodeType = self.getNodeTypeFromServer(server)
		node.setDataType(nodeType)
		
		# add managed node to the collection
		if( nodeType==Node.MANAGED_NODE ):
			self.managedNodeNum += 1
			node.managedNodeNum = self.managedNodeNum
			self.nodes.append(node.getNodeName())
		#endIf

		self.discoveredServers[server.getServerName()] = server
	#endDef
	
	"""
		Called by CellDefinitionReader when a cluster is discovered in the cell configuration
	"""	
	def clusterDiscovered(self, clusterId):
		self.clusterNum +=1
		isWpCluster = 0
		clusterName = AdminConfig.showAttribute(clusterId, 'name')
		self.LOGGER.log("CRWWA0143I", clusterName)
		self.clusters = self.clusters + clusterName + ","
		self.environmentDict["CLUSTER" + str(self.clusterNum)] = clusterName
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_CLUSTER_NAME"] = clusterName
		serverNodeMapping = {}
		serverNames = ArrayList()
		for clusterMember in AdminConfig.list('ClusterMember', clusterId).split(NEWLINE):
			if (len(clusterMember) > 0):
				serverName = AdminConfig.showAttribute(clusterMember, 'memberName')
				serverNames.add(serverName)
				nodeName = AdminConfig.showAttribute(clusterMember, 'nodeName')
				self.LOGGER.log("CRWWA0144I", [serverName, nodeName])
				if (not serverNodeMapping.has_key(nodeName)):
					serverNodeMapping[nodeName] = ArrayList()
				#endIf
				serverNodeMapping[nodeName].add(serverName)
				if (self.discoveredServers.has_key(serverName)):
					server = self.discoveredServers[serverName]
					if (server.getServerType() == Server.PORTAL_SERVER):
						isWpCluster = 1
					#endIf
				#endIf
			#endIf
		#endFor
		prefix = WsadminStringUtils.findCommonPrefix(serverNames)
		if (len(prefix) > 0):
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PREFIX"] = prefix
		else:
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PREFIX"] = clusterName + "_clone"
		#endIf
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_TRANSPORT_NODE_INCREMENTOR"] = "0"
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_TRANSPORT_STARTING_POINT"] = "0"
		
		nodes = ""
		maxPerNode=0
		for nodeName in serverNodeMapping.keys():
			nodes = nodes + nodeName + ","
			if (serverNodeMapping[nodeName].size() > maxPerNode):
				maxPerNode = serverNodeMapping[nodeName].size()
			#endIf
			if (isWpCluster and not self.environmentDict.has_key("CLUSTER" + str(self.clusterNum) + "_PRIMARY_NODE")):
				self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PRIMARY_NODE"] = nodeName
			#endIf
		#endFor
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_NODES"] = (nodes[0:len(nodes)-1])
		self.environmentDict["CLUSTER" + str(self.clusterNum) + "_PERNODE"] = str(maxPerNode)
		if (isWpCluster):
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_CLUSTER_TYPE"] = "WP"
			self.environmentDict["WP_CLUSTER_NAME"] = clusterName
		else:
			self.environmentDict["CLUSTER" + str(self.clusterNum) + "_CLUSTER_TYPE"] = "WAS"
		#endIf
	#endDef

	def _processAdditionalProperties(self):
		
		self.environmentDict["CELL_TYPE"] = self.cellType
		if (self.cellType == "WAS"):
			self.environmentDict["PRODUCT_VERSION"] = self.environmentDict["CELL_TYPE"] + self.environmentDict["WAS_VERSION"]
			#self.environmentDict["PRIMARY_NODE"] = ""
			#self.environmentDict["WP_VERSION"] = ""
		else:
			# cellType is portal
			if (self.environmentDict["WAS_VERSION"] == "60"):
				self.environmentDict["WP_VERSION"] = "60"
			elif (self.environmentDict["WAS_VERSION"] == "61"):
				self.environmentDict["WP_VERSION"] = "61"
			elif (self.environmentDict["WAS_VERSION"] == "70"):
				self.environmentDict["WP_VERSION"] = self._findWpVersion()
			elif (self.environmentDict["WAS_VERSION"] == "80"):
				self.environmentDict["WP_VERSION"] = "80"
			#endIf
			self.environmentDict["PRODUCT_VERSION"] = self.environmentDict["CELL_TYPE"] + self.environmentDict["WP_VERSION"]
		#endIf
			
		self.environmentDict["ENABLE_SECURITY_AT_CREATION"] = "true"
		self.environmentDict["USING_DB2"] = "false"
		self.environmentDict["DB2_CLIENT_HOME"] = ""
		self.environmentDict["ALLOW_NON_ROOT_SILENT_INSTALL"] = "true"
		
	#endDef
					
	"""
		Called by CellDefinitionReader after all topology objects have been discovered
		to gather the variables which have been extracted from the configuration
		
		These variables will be added to the env_cell.properties file.	
	"""	
	def getVariables(self):
		self.LOGGER.traceEnter()
		numberOfNodes = self.managedNodeNum
		wasVersion = ""
		
		## wasFullVersion is in the form 7.0.0.39
		wasFullVersion = ""
		for node in self.discoveredNodes.values():
			prevStandaloneVal = self.isStandalone
			if (node.nodeContainsAppServers):
				self.managedNodeNum = node.managedNodeNum
				if (not self.cellDefinitionReader.adminAgentManagedNodesDict.has_key(node.getNodeName())):
					self._processAppServerNode(node)
				else:
					self.isStandalone = 0
					self._processManagedNode(node)
				#endIf
				if (node.getWpPrimaryServer() != ""):
					self._setNodeProperty(node, "_PRIMARY_SERVER", node.getWpPrimaryServer())
					self._setNodeProperty(node, "_WP_SERVERS_ON_NODE", node.getWPServersOnNodeStr())
				#endIf
				self._setNodeProperty(node, "_SERVERS_ON_NODE", node.getServersOnNodeStr())
				self._setNodeProperty(node, "_STANDALONE_SERVERS_ON_NODE", node.getStandaloneServersOnNodeStr())
				self._setNodeProperty(node, "_LOAD_BALANCE_SERVERS_ON_NODE", node.getStandaloneServersOnNodeStr())
				self._setNodeProperty(node, "_NUMBER_OF_SERVERS", str(node.getNumServers()))
				self._setNodeProperty(node, "_NUMBER_OF_STANDALONE_SERVERS", str(node.getNumStandaloneServers()))
				self.isStandalone = prevStandaloneVal
			#endIf
			## add all server properties
			node.setServerProperties(self.environmentDict)
				
			## all nodes should have the same version
			## admin managed nodes are not defined in this wsadmin runtime, so just pull the attrs
			if( node.getDataType()==Node.ADMIN_MANAGED_NODE ):
				wasVersion = node.wasVersion
				nodeFullVersion = node.fullVersion
			else:
				wasVersion = node.getNodeVersion()
				nodeFullVersion = node.getNodeFullVersion()
			
			#endIf
			self._setNodeProperty(node, "_WAS_FULL_VERSION", str(nodeFullVersion))
			## take the largest version number in the cell 
			## (this will fail for things like 7.0.0.9 vs 7.0.0.19, but it rarely will happen)
			if (nodeFullVersion > wasFullVersion):
				wasFullVersion = nodeFullVersion
		#endFor
		self.environmentDict["WAS_VERSION"] = wasVersion
		self.environmentDict["WAS_FULL_VERSION"] = str(wasFullVersion)
		
		if (self.isStandalone):
			self.environmentDict["NODES"] = ""
			self.environmentDict["NUMBER_OF_NODES"] = str(0)
			self.environmentDict["IS_BASE"] = "yes"
			self.environmentDict["LOAD_BALANCE_NODES"] = ""
		else:
			self.environmentDict["NODES"] = ",".join(self.nodes)
			self.environmentDict["NUMBER_OF_NODES"] = numberOfNodes
			self.environmentDict["IS_BASE"] = "no"
			self.environmentDict["LOAD_BALANCE_NODES"] = ",".join(self.nodes)
		#endIf
		
		if (self.isAdminAgentCell == "true"):
			self.environmentDict["IS_BASE"] = "yes"
			self.environmentDict["IS_ADMIN_AGENT"] = "yes"
			self.environmentDict["NODES"] = ",".join(self.nodes)
			self.environmentDict["NUMBER_OF_ADMIN_MANAGED_NODES"] = numberOfNodes
			self.environmentDict["NUMBER_OF_NODES"] = numberOfNodes
			self.environmentDict["LOAD_BALANCE_NODES"] = ",".join(self.nodes)
		#endIf
		
		if (len(self.clusters) > 0):	
			self.environmentDict["CLUSTERS"] = (self.clusters[0:len(self.clusters)-1])
		else:
			self.environmentDict["CLUSTERS"] = ""
		#endIf
		self.environmentDict["NUMBER_OF_CLUSTERS"] = str(self.clusterNum)
		self._processAdditionalProperties()
		self.LOGGER.traceExit()
		return self.environmentDict
	#endDef
	
	"""
		Returns WebSphere Portal version. Default return value is 61.
		Check for the presence of 'wps_theme' application. If 'wps_theme'
		application is present set portal version to 70 and return
	"""	
	def _findWpVersion(self):
		wpVersion = "61"
		cells = AdminConfig.list('Cell').split(NEWLINE)
		for cell in cells:
			nodes = AdminConfig.list('Node', cell).split(NEWLINE)
			for node in nodes:
				cname = AdminConfig.showAttribute(cell, 'name')
				nname = AdminConfig.showAttribute(node, 'name')
				servs = AdminControl.queryNames('type=Server,cell=' + cname + ',node=' + nname + ',*').split(NEWLINE)
				for server in servs:
					if (server is None or server == ''):
						continue
					sname = AdminControl.getAttribute(server, 'name')
					if (sname == 'nodeagent'):
						continue
					apps = AdminControl.queryNames('type=Application,cell=' + cname + ',node=' + nname + ',process=' + sname + ',*').split(NEWLINE)
					for app in apps:
						if (app is None or app == ''):
							continue
						aname = AdminControl.getAttribute(app, 'name')
						if (aname == 'wps_theme'):
							wpVersion = "70"
							break
					#endFor
				#endFor
			#endFor
		#endFor
		return wpVersion
	#endDef
	
#endClass