'''
Licensed Materials - Property of IBM Corp. 
IBM Middleware Configuration for WebSphere 
(c) Copyright IBM Corporation 2003, 2013. All Rights Reserved.   

U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by 
GSA ADP Schedule Contract with IBM Corp. 


@author: Per Bergman
'''

import org.w3c.dom.Node as Node

from SimpleXPath import SimpleXPath
from OracleUtil import OracleUtil
from Analyzer import Analyzer

from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

class DepEnvAnalyzer(Analyzer):

    def __init__(self, file, collector, cellType, isOracle, bpmReader):
        self.LOGGER = _Logger("DepEnvAnalyzer", MessageManager.RB_WEBSPHERE_BPM)
        Analyzer.__init__(self, collector, cellType, isOracle, bpmReader)
        self.parser = SimpleXPath(file)
        self.numNodes = -1
        self.numServers = -1
        self.oraUtil = OracleUtil()
    #endDef
    
    def handleComponent(self, compId, prefix, defaultSchema, setDB):
        self.LOGGER.log("CRWBP0105I", compId)
        comp = self.parser.findOneByTag("components", "id", compId)
        ds = self.parser.findOneByTagAndNode(comp, "dataSrc", None, None)
        
        auth = self.parser.findOneByTagAndNode(ds, "authAlias", None, None)
        userName = self.parser.findAttrValue(auth, "userName")
        self.collect(prefix + "USER_NAME", userName)
        self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", self.parser.findAttrValue(auth, "password")))

        # Schema settings
        schema = self.parser.findAttrValue(ds, "schemaName")
        if (schema == None and defaultSchema != None):
            schema = defaultSchema
        
        # Oracle needs to have user name == schema name
        if(self.isOracleDB() == 1):
            schema = userName
        
        self.collect(prefix + "SCHEMA_NAME", schema)
        
        if (setDB == 1):
            if(self.isOracleDB() == 1):
                dbName = self.parser.findOneByTagAndNode(ds, "properties", "name", "URL")
                dbName, ignore = self.oraUtil.splitTNS(self.parser.findAttrValue(dbName, "value"));
                self.collect(prefix + "NAME", dbName)
            else:
                dbName = self.parser.findOneByTagAndNode(ds, "properties", "name", "databaseName")
                self.collect(prefix + "NAME", self.parser.findAttrValue(dbName, "value"))
    #endDef
    
    def process(self):
        pattern = self.parser.findOneByTag("pattern", None, None)
        self.collect("WBI_DEPLOYMENT_ENVIRONMENT_TOPOLOGY", self.parser.findAttrValue(pattern, "id"))
        self.collect("WBI_ENVIRONMENT_TYPE", "deploymentEnvironment")
        
        roles = self.parser.findByTag("roles", None, None)
        for role in roles:
            roleType = self.parser.findAttrValue(role, "name")
            self.LOGGER.log("CRWBP0106I", roleType)
            clusterName = self.parser.findOneByTagAndNode(role, "deploymentTarget", None, Node)
            cn = self.parser.findAttrValue(clusterName, "name")
            if (roleType == "AppTarget"):
                self.collect("WBI_APPLICATION_CLUSTER_NAME", cn)
            elif (roleType == "Support"):
                self.collect("WBI_SUPPORT_CLUSTER_NAME", cn)
            elif (roleType == "Messaging"):
                self.collect("WBI_MESSAGING_CLUSTER_NAME", cn)
            else:
                self.LOGGER.log("CRWBP0109I")
            #self.checkConsistency(role)
        #endFor
        
        # Don't care about schema name here
        self.handleComponent("WBI_CEI", "WBI_DB_CEI_", "", 0)
        if(self.cellType == "WPS"):
            self.handleComponent("WBI_BPC", "WPS_DB_BPC_", None, 1)
            self.handleComponent("WBI_BPCEventCollector", "WPS_DB_BPC_EVENT_", None, 0)
        
        # TODO this schema name is hardcoded
        #self.handleComponent("WBI_BSPACE", self.cellType + "_DB_BSPACE_", None, 0)
        self.handleComponent("WBI_BSPACE", "WBI_DB_BSPACE_", None, 0)
        
        self.handleBus("WBI_CEI", "WBI_DB_CEI_ME_", "CEI_bus")
        self.handleBus("WBI_SCA", "WBI_DB_SCA_SYS_ME_", "SCA_system_bus")
        self.handleBus("WBI_SCA", "WBI_DB_SCA_APP_ME_", "SCA_app_bus")
        
        if(self.cellType == "WPS"):
            self.handleBus("WBI_BPC", "WPS_DB_BPC_ME_", "BPC_bus")
        self.handleJMSAuth("WBI_CEI", "WBI_JMS_", "CommonEventInfrastructureJMSAuthAlias")
        
        if(self.cellType == "WPS"):
            self.handleJMSAuth("WBI_BPC", "WPS_JMS_BPC_", "BPC_Auth_Alias")
    #endDef
    
    def handleBus(self, compId, prefix, bus):
        self.LOGGER.log("CRWBP0107I", compId);
        comp = self.parser.findOneByTag("components", "id", compId)
        busNode = self.parser.findOneByTagAndNode(comp, "siBuses", "name", bus)
        
        ds = self.parser.findOneByTagAndNode(busNode, "dataSrc", None, None)
        schema = self.parser.findAttrValue(ds, "schemaName")
        self.collect(prefix + "SCHEMA_NAME", schema)
        
        auth = self.parser.findOneByTagAndNode(busNode, "authAlias", None, None)
        self.collect(prefix + "USER_NAME", self.parser.findAttrValue(auth, "userName"))
        self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", self.parser.findAttrValue(auth, "password")))
    #endDef
    
    def handleJMSAuth(self, compId, prefix, alias):
        self.LOGGER.log("CRWBP0108I", compId)
        comp = self.parser.findOneByTag("components", "id", compId)
        auth = self.parser.findOneByTagAndNode(comp, "authAliases", "name", alias)
        self.collect(prefix + "USER", self.parser.findAttrValue(auth, "userName"))
        self.collect(prefix + "GROUP", "")
        self.collect(prefix + "PASSWORD", self.encrypt(prefix + "PASSWORD", self.parser.findAttrValue(auth, "password")))
    #endDef
        
#endClass