'''
Created on Nov 11, 2010

@author: Per Bergman
'''

from    java.lang import System
from    java.io import StringReader

from    org.python.modules import re
import  org.xml.sax as sax
import  org.apache.xerces.parsers.DOMParser as domparser
import  org.w3c.dom.traversal.NodeFilter as NodeFilter
import  org.w3c.dom.traversal as traversal
import  org.w3c.dom.Node as Node
import  org.w3c.dom.Document as Document

from org.w3c.dom import NodeList
from Logger import _Logger
from com.ibm.rational.rafw.wsadmin.logging import MessageManager

'''
Implements a few XPath like methods
'''
class SimpleXPath:

    def __init__(self, file):
        self.LOGGER = _Logger("SimpleXPath", MessageManager.RB_WEBSPHERE_WAS)
        parser = domparser()

        try:
            document = open(file).read()
        except IOError:
            print file + " does not exist"
            return

        parser.reset()
        documentIS = sax.InputSource(StringReader(document))
        parser.parse(documentIS)
        self.root = parser.getDocument()
    #endDef

    """
    Looks for an XML element recursively from the root, optionally
    having an attribute "name" with a "value".
    E.g. "findByTag(self, "components", "id", someId)"

    Returns a list of Node.
    """
    def findByTag(self, tag, name, value):
        ret = []
        elts = self.root.getElementsByTagName(tag)
        for i in range(elts.getLength()):
            current = elts.item(i)
            if (name != None):
                aValue = self.findAttrValue(current, name)
                if (aValue == value):
                    ret.append(current)
            else:
                ret.append(current)
        return ret
    #endDef

    """
    Like 'findByTag' returning only the first Node.
    """
    def findOneByTag(self, tag, name, value):
        found = self.findByTag(tag, name, value)
        if (len(found) > 0):
            return found[0]
        return None
    #endDef

    """
    Looks for an XML element recursively from the 'owner' Node,
    optionally having an attribute "name" with a "value".
    E.g. "findByTag(self, owner, "components", "id", someId)"

    Returns a list of Node.
    """
    def findByTagAndNode(self, owner, tag, name, value):
        ret = []
        kids = owner.getChildNodes()
        for i in range(kids.getLength()):
            current = kids.item(i)
            if (current.getNodeName() == tag):
                if (name != None):
                    aValue = self.findAttrValue(current, name)
                    if (aValue == value):
                        ret.append(current)
                else:
                    ret.append(current)
            ret.extend(self.findByTagAndNode(current, tag, name, value))
        return ret
    #endDef

    """
    Like 'findByTagAndNode' returning only the first Node.
    """
    def findOneByTagAndNode(self, owner, tag, name, value):
        found = self.findByTagAndNode(owner, tag, name, value)
        if (len(found) > 0):
            return found[0]
        return None
    #endDef

    """
    Returns a given attribute 'name'.
    """
    def findAttr(self, owner, name):
        return owner.getAttributes().getNamedItem(name)
    #endDef

    """
    Returns a the value of given attribute name.
    """
    def findAttrValue(self, owner, name):
        node = self.findAttr(owner, name)
        if (node != None):
            return node.getNodeValue()
        return None
    #endDef

    def dump(self, nl):
        map(self.dumpNode, nl)
#        for i in range(nl.size()):
#            self.dumpNode(nl[i])
    #endDef

    def dumpNode(self, node):
        for a in self.nodifyAttributes(node):
            print(a.getNodeName() + "=" + a.getNodeValue() + ", ")
        print
    #endDef

    def nodifyAttributes(self, n):
        ret = []
        attrs = n.getAttributes()
        for i in range(attrs.getLength()):
            ret.append(attrs.item(i))
        return ret
    #endDef

#endClass
