-----------------------------------------------
-- Licensed Materials - Property of IBM
--
-- 5724-R31, 5655-S30
--
-- (C) Copyright IBM Corp. 2006, 2008 All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
------------------------------------------------



-- ###############################################
-- WARNING
-- Do not perform any committable statements in this file
-- as they will be rolled back in the case where the user
-- drops the W_ tables and expects the system to bootstrap
-- the tables again but did not drop the sequences.  When
-- the CREATE SEQUENCE statements fail, the transation will
-- be rolled back along with any committable work.
-----------------------------------------------------------
----Create tables ,primary keys & indexes for  triplestore
-------------------------------------------------------------
 
CREATE TABLE w_lit_double (
    id BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY(START WITH 1,INCREMENT BY 1), 
    litval DOUBLE NOT NULL,
    UNIQUE (litval))   
 IN USERSPACE1; 

ALTER TABLE w_lit_double ADD PRIMARY KEY (id);

CREATE INDEX idx_obj_dbl ON w_lit_double (litval, id);

CREATE TABLE w_lit_float (
    id BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY(START WITH 1,INCREMENT BY 1), 
    litval REAL NOT NULL,
    UNIQUE (litval))  
 IN USERSPACE1; 

ALTER TABLE w_lit_float ADD PRIMARY KEY (id);

CREATE INDEX idx_obj_flt ON w_lit_float (litval, id);

CREATE TABLE w_obj_lit_any (
    id BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY(START WITH 1,INCREMENT BY 1), 
    large LONG VARCHAR FOR BIT DATA, 
    hash CHAR(32) NOT NULL, 
    type_uri VARCHAR(254) NOT NULL,
    UNIQUE (hash))  
 IN USERSPACE1; 

ALTER TABLE w_obj_lit_any ADD PRIMARY KEY (id);

CREATE INDEX idx_obj_any ON w_obj_lit_any (hash, id);

CREATE TABLE w_schema_rev (
	rev INTEGER NOT NULL,
	PRIMARY KEY(rev) )
 IN USERSPACE1;

DELETE FROM w_schema_rev;
INSERT INTO w_schema_rev (rev) VALUES (0);

CREATE TABLE w_mod_lock (
    id INTEGER NOT NULL,
	mlock INTEGER NOT NULL )
 IN USERSPACE1;

DELETE FROM w_mod_lock;
INSERT INTO w_mod_lock (id,mlock) VALUES (1,0);


CREATE TABLE w_obj_lit_plain (
    id BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY(START WITH 1,INCREMENT BY 1),
    large LONG VARCHAR FOR BIT DATA,
    litval VARCHAR(1024),
    hash CHAR(32) NOT NULL,
    fullhash CHAR(32),
    xml_lang VARCHAR(64),
	typedstr SMALLINT)
 IN USERSPACE1;

ALTER TABLE w_obj_lit_plain ADD PRIMARY KEY (id);

CREATE INDEX idx_obj_plain_hash ON w_obj_lit_plain (hash, typedstr, id);
CREATE INDEX idx_obj_plain ON w_obj_lit_plain (fullhash, id);

CREATE TABLE w_uri (
    id BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY(START WITH 1,INCREMENT BY 1), 
    litval VARCHAR(1020) NOT NULL, 
    hash CHAR(32) NOT NULL,
    namespace_id INTEGER,
    UNIQUE (hash) )  
 IN USERSPACE1; 

ALTER TABLE w_uri ADD PRIMARY KEY (id);

CREATE INDEX idx_uri ON w_uri (hash, id);

CREATE TABLE w_statement (
    subj_id BIGINT NOT NULL, 
    pred_id BIGINT NOT NULL, 
    obj_id BIGINT NOT NULL, 
    obj_typ_cd SMALLINT NOT NULL)  
 IN USERSPACE1; 

--distribution likely to be *very* good for statement - use as clustering index only?
CREATE INDEX idx_clust_sbj ON w_statement (subj_id) CLUSTER;
CREATE INDEX idx_smt_by_obj ON w_statement (obj_id, obj_typ_cd, pred_id, subj_id) ALLOW REVERSE SCANS;
CREATE INDEX idx_smt_by_prd ON w_statement (pred_id, obj_id, obj_typ_cd, subj_id) ALLOW REVERSE SCANS;
CREATE INDEX idx_smt_by_sbj ON w_statement (subj_id ASC, pred_id ASC, obj_typ_cd ASC, obj_id ASC) ALLOW REVERSE SCANS;
CREATE INDEX idx_smt_by_typ ON w_statement (obj_typ_cd ASC, obj_id ASC, pred_id ASC, subj_id ASC) ALLOW REVERSE SCANS;

CREATE TABLE w_artifact_blob (
 id VARCHAR(254) NOT NULL, 
 content BLOB(1G), 
 deleted SMALLINT,
 PRIMARY KEY(id))
IN USERSPACE1; 
		
---------------------------------------------------------------------------------
-----Create Foreign keys on trplestore tables
---------------------------------------------------------------------------------

ALTER TABLE w_uri 
	ADD CONSTRAINT w_uri_fk_namesp FOREIGN KEY
		(namespace_id)
	REFERENCES w_uri
		(id)
	ON DELETE NO ACTION
	ON UPDATE NO ACTION
	ENFORCED
	ENABLE QUERY OPTIMIZATION;

CREATE TABLE w_dbversion (
    subsystem VARCHAR(8) NOT NULL, 
    cur_version INTEGER NOT NULL )  
 IN USERSPACE1;

ALTER TABLE w_dbversion ADD PRIMARY KEY (subsystem);

ALTER TABLE w_statement VOLATILE;
ALTER TABLE w_uri VOLATILE;
ALTER TABLE w_artifact_blob VOLATILE;
ALTER TABLE w_obj_lit_plain VOLATILE;
ALTER TABLE w_lit_double VOLATILE;
ALTER TABLE w_lit_float VOLATILE;
ALTER TABLE w_obj_lit_any VOLATILE;

GRANT SELECT,INSERT,UPDATE ON w_dbversion TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE,DELETE ON w_statement TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_uri TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE,DELETE ON w_artifact_blob TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_obj_lit_plain TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_lit_double TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_lit_float TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_obj_lit_any TO USER @DBUSER@;
GRANT SELECT,INSERT,UPDATE ON w_schema_rev TO USER @DBUSER@;
GRANT UPDATE ON w_mod_lock TO USER @DBUSER@;

--#SET TERMINATOR @
---------------------------------------------------------------------
---- Stored procedure for inserts to W_OBJ_LIT_PLAIN
---------------------------------------------------------------------
CREATE PROCEDURE PMAPSTR (IN hash_in     CHAR(32),
                          IN fullhash_in CHAR(32),
                          IN xmllang_in  VARCHAR(64),
                          IN litval_in   VARCHAR(1024),
                          IN large_in    VARCHAR(32672) FOR BIT DATA, 
                          IN typedstr_in SMALLINT,
                          OUT result     BIGINT) 
                          LANGUAGE SQL MODIFIES SQL DATA
UPDATEPTAB: BEGIN
   IF typedstr_in = 1 THEN
   	IF (NOT EXISTS( SELECT id from w_obj_lit_plain where hash = hash_in AND typedstr = 1)) THEN
     		INSERT INTO w_obj_lit_plain
        		( LARGE, LITVAL, HASH, TYPEDSTR ) 
        	VALUES 
        		( large_in, litval_in, hash_in, typedstr_in );
   	END IF;
   	SET result = (SELECT id from w_obj_lit_plain where hash = hash_in AND typedstr = 1);
   ELSE 
   	IF (NOT EXISTS( SELECT id from w_obj_lit_plain where fullhash = fullhash_in)) THEN
     		INSERT INTO w_obj_lit_plain
        		( LARGE, LITVAL, HASH, FULLHASH, XML_LANG, TYPEDSTR ) 
        	VALUES 
        		( large_in, litval_in, hash_in, fullhash_in, xmllang_in, typedstr_in);
   	END IF;
   	SET result = (SELECT id from w_obj_lit_plain where fullhash = fullhash_in);
   END IF;
END UPDATEPTAB@
---------------------------------------------------------------------
---- Stored procedure for inserts to W_URI 
---------------------------------------------------------------------
CREATE PROCEDURE PMAPURI (IN hash_in         CHAR(32),
                          IN litval_in       VARCHAR(1020),
                          IN namespace_id_in BIGINT,
                          OUT result         BIGINT) 
                          LANGUAGE SQL MODIFIES SQL DATA
UPDATEURITAB: BEGIN
   IF (NOT EXISTS( SELECT id FROM w_uri WHERE hash=hash_in)) THEN
      INSERT INTO w_uri( LITVAL, HASH, NAMESPACE_ID ) 
      	VALUES ( litval_in, hash_in, NULLIF(namespace_id_in, -1));
   END IF;
   SET result = (SELECT id FROM w_uri WHERE hash=hash_in);
END UPDATEURITAB@

GRANT EXECUTE ON PROCEDURE PMAPURI TO USER @DBUSER@ @
GRANT EXECUTE ON PROCEDURE PMAPSTR TO USER @DBUSER@ @
